{-# OPTIONS_GHC -w #-}
module Parser where

import Data.Char
import Processes
import Expressions
import Auxiliary
import DataSpec
import Debug.Trace

printExpressions [] = []
printExpressions updates = infixString [(printExpression var) ++ " := " ++ (printExpression val) | (var,val) <- updates] ", "

data Item = DataActions [(String, ActionType)]
          | DataHiding [String] 
          | DataGlobal (Variable, Type) Expression 
          | DataRenaming [(String, String)] 
          | DataUntilFormula String
          | DataFormula Expression
          | DataEncapsulation [String]   
          | DataNoCommunication [String]   
          | DataReach [String]   
          | DataReachCondition Expression   
          | DataCommunication [(String, String, String)] 
          | DataConstant String Expression
          | DataEnumType String [String] 
          | DataNat String
          | DataQueue String
          | DataRangeType String Expression Expression
          | DataFunction String [([String], String)]
          | ParserProcess Process
          | ParserInitialProcess InitialProcessDefinition
              deriving (Show, Eq)

data ActionType = None | Bool | IntRange Expression Expression deriving (Show, Eq)

data InitialProcessDefinition = InitSingleProcess InitialProcess
                              | InitParallel InitialProcessDefinition InitialProcessDefinition
                              | InitHiding [Action] InitialProcessDefinition
                              | InitEncapsulation [Action] InitialProcessDefinition
                              | InitRenaming [(Action, Action)] InitialProcessDefinition
                                   deriving (Show, Eq)

type LineNumber = Int
data ParseResult a = Ok a | Failed String
type ParserMonad a = String -> LineNumber -> ParseResult a

getLineNo :: ParserMonad LineNumber
getLineNo = \s l -> Ok l


thenParserMonad :: ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
m `thenParserMonad ` k = \s -> \l ->
   case (m s l) of 
       Ok a -> k a s l
       Failed e -> Failed e

returnParserMonad :: a -> ParserMonad a
returnParserMonad a = \s -> \l -> Ok a

failParserMonad :: String -> ParserMonad a
failParserMonad err = \s -> \l -> Failed (err ++ " on line " ++ show l)

-- parser produced by Happy Version 1.18.9

data HappyAbsSyn t4 t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 t21 t22 t23 t24 t25 t26 t27 t28 t29 t30 t31 t32 t33 t34 t35 t36 t37 t38
	= HappyTerminal (Token)
	| HappyErrorToken Int
	| HappyAbsSyn4 t4
	| HappyAbsSyn5 t5
	| HappyAbsSyn6 t6
	| HappyAbsSyn7 t7
	| HappyAbsSyn8 t8
	| HappyAbsSyn9 t9
	| HappyAbsSyn10 t10
	| HappyAbsSyn11 t11
	| HappyAbsSyn12 t12
	| HappyAbsSyn13 t13
	| HappyAbsSyn14 t14
	| HappyAbsSyn15 t15
	| HappyAbsSyn16 t16
	| HappyAbsSyn17 t17
	| HappyAbsSyn18 t18
	| HappyAbsSyn19 t19
	| HappyAbsSyn20 t20
	| HappyAbsSyn21 t21
	| HappyAbsSyn22 t22
	| HappyAbsSyn23 t23
	| HappyAbsSyn24 t24
	| HappyAbsSyn25 t25
	| HappyAbsSyn26 t26
	| HappyAbsSyn27 t27
	| HappyAbsSyn28 t28
	| HappyAbsSyn29 t29
	| HappyAbsSyn30 t30
	| HappyAbsSyn31 t31
	| HappyAbsSyn32 t32
	| HappyAbsSyn33 t33
	| HappyAbsSyn34 t34
	| HappyAbsSyn35 t35
	| HappyAbsSyn36 t36
	| HappyAbsSyn37 t37
	| HappyAbsSyn38 t38

action_0 (66) = happyShift action_4
action_0 (71) = happyShift action_5
action_0 (72) = happyShift action_6
action_0 (73) = happyShift action_7
action_0 (75) = happyShift action_8
action_0 (76) = happyShift action_9
action_0 (77) = happyShift action_10
action_0 (78) = happyShift action_11
action_0 (79) = happyShift action_12
action_0 (80) = happyShift action_13
action_0 (81) = happyShift action_14
action_0 (82) = happyShift action_15
action_0 (83) = happyShift action_16
action_0 (84) = happyShift action_17
action_0 (85) = happyShift action_18
action_0 (86) = happyShift action_19
action_0 (4) = happyGoto action_20
action_0 (5) = happyGoto action_2
action_0 (6) = happyGoto action_3
action_0 _ = happyReduce_2

action_1 (66) = happyShift action_4
action_1 (71) = happyShift action_5
action_1 (72) = happyShift action_6
action_1 (73) = happyShift action_7
action_1 (75) = happyShift action_8
action_1 (76) = happyShift action_9
action_1 (77) = happyShift action_10
action_1 (78) = happyShift action_11
action_1 (79) = happyShift action_12
action_1 (80) = happyShift action_13
action_1 (81) = happyShift action_14
action_1 (82) = happyShift action_15
action_1 (83) = happyShift action_16
action_1 (84) = happyShift action_17
action_1 (85) = happyShift action_18
action_1 (86) = happyShift action_19
action_1 (5) = happyGoto action_2
action_1 (6) = happyGoto action_3
action_1 _ = happyFail

action_2 _ = happyReduce_1

action_3 (66) = happyShift action_4
action_3 (71) = happyShift action_5
action_3 (72) = happyShift action_6
action_3 (73) = happyShift action_7
action_3 (75) = happyShift action_8
action_3 (76) = happyShift action_9
action_3 (77) = happyShift action_10
action_3 (78) = happyShift action_11
action_3 (79) = happyShift action_12
action_3 (80) = happyShift action_13
action_3 (81) = happyShift action_14
action_3 (82) = happyShift action_15
action_3 (83) = happyShift action_16
action_3 (84) = happyShift action_17
action_3 (85) = happyShift action_18
action_3 (86) = happyShift action_19
action_3 (5) = happyGoto action_57
action_3 (6) = happyGoto action_3
action_3 _ = happyReduce_2

action_4 (46) = happyShift action_52
action_4 (71) = happyShift action_53
action_4 (72) = happyShift action_54
action_4 (73) = happyShift action_55
action_4 (76) = happyShift action_56
action_4 (20) = happyGoto action_51
action_4 _ = happyFail

action_5 (46) = happyShift action_49
action_5 (68) = happyShift action_50
action_5 _ = happyFail

action_6 (71) = happyShift action_37
action_6 (17) = happyGoto action_48
action_6 _ = happyFail

action_7 (46) = happyShift action_47
action_7 (15) = happyGoto action_45
action_7 (16) = happyGoto action_46
action_7 _ = happyFail

action_8 (71) = happyShift action_44
action_8 (8) = happyGoto action_42
action_8 (9) = happyGoto action_43
action_8 _ = happyFail

action_9 (71) = happyShift action_37
action_9 (17) = happyGoto action_41
action_9 _ = happyFail

action_10 (46) = happyShift action_40
action_10 (13) = happyGoto action_38
action_10 (14) = happyGoto action_39
action_10 _ = happyFail

action_11 (71) = happyShift action_37
action_11 (17) = happyGoto action_36
action_11 _ = happyFail

action_12 (62) = happyShift action_34
action_12 (71) = happyShift action_35
action_12 (18) = happyGoto action_32
action_12 (19) = happyGoto action_33
action_12 _ = happyFail

action_13 (46) = happyShift action_26
action_13 (55) = happyShift action_27
action_13 (57) = happyShift action_28
action_13 (71) = happyShift action_29
action_13 (37) = happyGoto action_31
action_13 _ = happyFail

action_14 (71) = happyShift action_30
action_14 _ = happyFail

action_15 (46) = happyShift action_26
action_15 (55) = happyShift action_27
action_15 (57) = happyShift action_28
action_15 (71) = happyShift action_29
action_15 (37) = happyGoto action_25
action_15 _ = happyFail

action_16 (71) = happyShift action_24
action_16 _ = happyFail

action_17 (44) = happyShift action_23
action_17 _ = happyFail

action_18 (71) = happyShift action_22
action_18 _ = happyFail

action_19 (71) = happyShift action_21
action_19 _ = happyFail

action_20 (89) = happyAccept
action_20 _ = happyFail

action_21 (68) = happyShift action_113
action_21 _ = happyFail

action_22 (68) = happyShift action_112
action_22 _ = happyFail

action_23 (41) = happyShift action_106
action_23 (46) = happyShift action_107
action_23 (47) = happyShift action_108
action_23 (50) = happyShift action_109
action_23 (55) = happyShift action_110
action_23 (71) = happyShift action_111
action_23 (7) = happyGoto action_105
action_23 _ = happyReduce_22

action_24 (60) = happyShift action_104
action_24 _ = happyFail

action_25 (47) = happyShift action_86
action_25 (51) = happyShift action_87
action_25 (52) = happyShift action_88
action_25 (54) = happyShift action_89
action_25 (56) = happyShift action_90
action_25 (57) = happyShift action_91
action_25 (58) = happyShift action_92
action_25 (59) = happyShift action_93
action_25 (62) = happyShift action_94
action_25 (63) = happyShift action_95
action_25 (65) = happyShift action_96
action_25 (67) = happyShift action_97
action_25 (68) = happyShift action_98
action_25 _ = happyReduce_14

action_26 (46) = happyShift action_26
action_26 (55) = happyShift action_27
action_26 (57) = happyShift action_28
action_26 (71) = happyShift action_29
action_26 (37) = happyGoto action_103
action_26 _ = happyFail

action_27 (46) = happyShift action_26
action_27 (55) = happyShift action_27
action_27 (57) = happyShift action_28
action_27 (71) = happyShift action_29
action_27 (37) = happyGoto action_102
action_27 _ = happyFail

action_28 (46) = happyShift action_26
action_28 (55) = happyShift action_27
action_28 (57) = happyShift action_28
action_28 (71) = happyShift action_29
action_28 (37) = happyGoto action_101
action_28 _ = happyFail

action_29 (46) = happyShift action_100
action_29 _ = happyReduce_116

action_30 (68) = happyShift action_99
action_30 _ = happyFail

action_31 (47) = happyShift action_86
action_31 (51) = happyShift action_87
action_31 (52) = happyShift action_88
action_31 (54) = happyShift action_89
action_31 (56) = happyShift action_90
action_31 (57) = happyShift action_91
action_31 (58) = happyShift action_92
action_31 (59) = happyShift action_93
action_31 (62) = happyShift action_94
action_31 (63) = happyShift action_95
action_31 (65) = happyShift action_96
action_31 (67) = happyShift action_97
action_31 (68) = happyShift action_98
action_31 _ = happyReduce_12

action_32 _ = happyReduce_11

action_33 (41) = happyShift action_85
action_33 _ = happyReduce_47

action_34 (62) = happyShift action_34
action_34 (71) = happyShift action_35
action_34 (19) = happyGoto action_84
action_34 _ = happyFail

action_35 (46) = happyShift action_83
action_35 _ = happyReduce_49

action_36 _ = happyReduce_10

action_37 (41) = happyShift action_82
action_37 _ = happyReduce_45

action_38 _ = happyReduce_9

action_39 (41) = happyShift action_81
action_39 _ = happyReduce_39

action_40 (71) = happyShift action_80
action_40 _ = happyFail

action_41 _ = happyReduce_8

action_42 _ = happyReduce_6

action_43 (41) = happyShift action_79
action_43 _ = happyReduce_29

action_44 (60) = happyShift action_78
action_44 _ = happyReduce_31

action_45 _ = happyReduce_5

action_46 (41) = happyShift action_77
action_46 _ = happyReduce_42

action_47 (71) = happyShift action_76
action_47 _ = happyFail

action_48 _ = happyReduce_4

action_49 (71) = happyShift action_75
action_49 (28) = happyGoto action_73
action_49 (29) = happyGoto action_74
action_49 _ = happyFail

action_50 (39) = happyShift action_69
action_50 (46) = happyShift action_70
action_50 (55) = happyShift action_27
action_50 (57) = happyShift action_28
action_50 (62) = happyShift action_71
action_50 (71) = happyShift action_72
action_50 (23) = happyGoto action_66
action_50 (24) = happyGoto action_67
action_50 (37) = happyGoto action_68
action_50 _ = happyFail

action_51 (43) = happyShift action_65
action_51 _ = happyReduce_19

action_52 (46) = happyShift action_52
action_52 (71) = happyShift action_53
action_52 (72) = happyShift action_54
action_52 (73) = happyShift action_55
action_52 (76) = happyShift action_56
action_52 (20) = happyGoto action_64
action_52 _ = happyFail

action_53 (44) = happyShift action_62
action_53 (46) = happyShift action_63
action_53 (22) = happyGoto action_61
action_53 _ = happyReduce_62

action_54 (46) = happyShift action_60
action_54 _ = happyFail

action_55 (46) = happyShift action_59
action_55 _ = happyFail

action_56 (46) = happyShift action_58
action_56 _ = happyFail

action_57 _ = happyReduce_3

action_58 (71) = happyShift action_37
action_58 (17) = happyGoto action_176
action_58 _ = happyFail

action_59 (46) = happyShift action_47
action_59 (15) = happyGoto action_175
action_59 (16) = happyGoto action_46
action_59 _ = happyFail

action_60 (71) = happyShift action_37
action_60 (17) = happyGoto action_174
action_60 _ = happyFail

action_61 _ = happyReduce_52

action_62 (45) = happyShift action_173
action_62 (46) = happyShift action_26
action_62 (55) = happyShift action_27
action_62 (57) = happyShift action_28
action_62 (71) = happyShift action_29
action_62 (36) = happyGoto action_172
action_62 (37) = happyGoto action_129
action_62 _ = happyFail

action_63 (71) = happyShift action_171
action_63 (32) = happyGoto action_169
action_63 (33) = happyGoto action_170
action_63 _ = happyFail

action_64 (43) = happyShift action_65
action_64 (50) = happyShift action_168
action_64 _ = happyFail

action_65 (46) = happyShift action_52
action_65 (71) = happyShift action_53
action_65 (72) = happyShift action_54
action_65 (73) = happyShift action_55
action_65 (76) = happyShift action_56
action_65 (20) = happyGoto action_167
action_65 _ = happyFail

action_66 (53) = happyShift action_166
action_66 _ = happyReduce_21

action_67 _ = happyReduce_75

action_68 (47) = happyShift action_86
action_68 (51) = happyShift action_87
action_68 (52) = happyShift action_88
action_68 (54) = happyShift action_89
action_68 (56) = happyShift action_90
action_68 (57) = happyShift action_91
action_68 (58) = happyShift action_92
action_68 (59) = happyShift action_93
action_68 (62) = happyShift action_94
action_68 (63) = happyShift action_95
action_68 (65) = happyShift action_96
action_68 (67) = happyShift action_97
action_68 (68) = happyShift action_98
action_68 (69) = happyShift action_165
action_68 _ = happyFail

action_69 (46) = happyShift action_164
action_69 _ = happyFail

action_70 (39) = happyShift action_69
action_70 (46) = happyShift action_70
action_70 (55) = happyShift action_27
action_70 (57) = happyShift action_28
action_70 (62) = happyShift action_71
action_70 (71) = happyShift action_72
action_70 (23) = happyGoto action_162
action_70 (24) = happyGoto action_67
action_70 (37) = happyGoto action_163
action_70 _ = happyFail

action_71 (46) = happyShift action_26
action_71 (55) = happyShift action_27
action_71 (57) = happyShift action_28
action_71 (71) = happyShift action_29
action_71 (37) = happyGoto action_161
action_71 _ = happyFail

action_72 (44) = happyShift action_159
action_72 (46) = happyShift action_160
action_72 (47) = happyReduce_116
action_72 (50) = happyReduce_116
action_72 (51) = happyReduce_116
action_72 (52) = happyReduce_116
action_72 (54) = happyReduce_116
action_72 (56) = happyReduce_116
action_72 (57) = happyReduce_116
action_72 (58) = happyReduce_116
action_72 (59) = happyReduce_116
action_72 (62) = happyReduce_116
action_72 (63) = happyReduce_116
action_72 (65) = happyReduce_116
action_72 (67) = happyReduce_116
action_72 (68) = happyReduce_116
action_72 (69) = happyReduce_116
action_72 (70) = happyReduce_116
action_72 (21) = happyGoto action_158
action_72 _ = happyReduce_59

action_73 (41) = happyShift action_156
action_73 (50) = happyShift action_157
action_73 _ = happyFail

action_74 _ = happyReduce_86

action_75 (60) = happyShift action_155
action_75 _ = happyFail

action_76 (41) = happyShift action_154
action_76 _ = happyFail

action_77 (46) = happyShift action_47
action_77 (15) = happyGoto action_153
action_77 (16) = happyGoto action_46
action_77 _ = happyFail

action_78 (48) = happyShift action_151
action_78 (74) = happyShift action_152
action_78 _ = happyFail

action_79 (71) = happyShift action_44
action_79 (8) = happyGoto action_150
action_79 (9) = happyGoto action_43
action_79 _ = happyFail

action_80 (41) = happyShift action_149
action_80 _ = happyFail

action_81 (46) = happyShift action_40
action_81 (13) = happyGoto action_148
action_81 (14) = happyGoto action_39
action_81 _ = happyFail

action_82 (71) = happyShift action_37
action_82 (17) = happyGoto action_147
action_82 _ = happyFail

action_83 (71) = happyShift action_37
action_83 (17) = happyGoto action_146
action_83 _ = happyFail

action_84 (63) = happyShift action_145
action_84 _ = happyFail

action_85 (62) = happyShift action_34
action_85 (71) = happyShift action_35
action_85 (18) = happyGoto action_144
action_85 (19) = happyGoto action_33
action_85 _ = happyFail

action_86 (46) = happyShift action_26
action_86 (55) = happyShift action_27
action_86 (57) = happyShift action_28
action_86 (71) = happyShift action_29
action_86 (37) = happyGoto action_143
action_86 _ = happyFail

action_87 (46) = happyShift action_26
action_87 (55) = happyShift action_27
action_87 (57) = happyShift action_28
action_87 (71) = happyShift action_29
action_87 (37) = happyGoto action_142
action_87 _ = happyFail

action_88 (46) = happyShift action_26
action_88 (55) = happyShift action_27
action_88 (57) = happyShift action_28
action_88 (71) = happyShift action_29
action_88 (37) = happyGoto action_141
action_88 _ = happyFail

action_89 (46) = happyShift action_26
action_89 (55) = happyShift action_27
action_89 (57) = happyShift action_28
action_89 (71) = happyShift action_29
action_89 (37) = happyGoto action_140
action_89 _ = happyFail

action_90 (46) = happyShift action_26
action_90 (55) = happyShift action_27
action_90 (57) = happyShift action_28
action_90 (71) = happyShift action_29
action_90 (37) = happyGoto action_139
action_90 _ = happyFail

action_91 (46) = happyShift action_26
action_91 (55) = happyShift action_27
action_91 (57) = happyShift action_28
action_91 (71) = happyShift action_29
action_91 (37) = happyGoto action_138
action_91 _ = happyFail

action_92 (46) = happyShift action_26
action_92 (55) = happyShift action_27
action_92 (57) = happyShift action_28
action_92 (71) = happyShift action_29
action_92 (37) = happyGoto action_137
action_92 _ = happyFail

action_93 (46) = happyShift action_26
action_93 (55) = happyShift action_27
action_93 (57) = happyShift action_28
action_93 (71) = happyShift action_29
action_93 (37) = happyGoto action_136
action_93 _ = happyFail

action_94 (46) = happyShift action_26
action_94 (55) = happyShift action_27
action_94 (57) = happyShift action_28
action_94 (71) = happyShift action_29
action_94 (37) = happyGoto action_135
action_94 _ = happyFail

action_95 (46) = happyShift action_26
action_95 (55) = happyShift action_27
action_95 (57) = happyShift action_28
action_95 (71) = happyShift action_29
action_95 (37) = happyGoto action_134
action_95 _ = happyFail

action_96 (46) = happyShift action_26
action_96 (55) = happyShift action_27
action_96 (57) = happyShift action_28
action_96 (71) = happyShift action_29
action_96 (37) = happyGoto action_133
action_96 _ = happyFail

action_97 (46) = happyShift action_26
action_97 (55) = happyShift action_27
action_97 (57) = happyShift action_28
action_97 (71) = happyShift action_29
action_97 (37) = happyGoto action_132
action_97 _ = happyFail

action_98 (46) = happyShift action_26
action_98 (55) = happyShift action_27
action_98 (57) = happyShift action_28
action_98 (71) = happyShift action_29
action_98 (37) = happyGoto action_131
action_98 _ = happyFail

action_99 (46) = happyShift action_26
action_99 (55) = happyShift action_27
action_99 (57) = happyShift action_28
action_99 (71) = happyShift action_29
action_99 (37) = happyGoto action_130
action_99 _ = happyFail

action_100 (46) = happyShift action_26
action_100 (55) = happyShift action_27
action_100 (57) = happyShift action_28
action_100 (71) = happyShift action_29
action_100 (36) = happyGoto action_128
action_100 (37) = happyGoto action_129
action_100 _ = happyFail

action_101 _ = happyReduce_112

action_102 (51) = happyShift action_87
action_102 (52) = happyShift action_88
action_102 (56) = happyShift action_90
action_102 (57) = happyShift action_91
action_102 (58) = happyShift action_92
action_102 (59) = happyShift action_93
action_102 (62) = happyShift action_94
action_102 (63) = happyShift action_95
action_102 (65) = happyShift action_96
action_102 (67) = happyShift action_97
action_102 (68) = happyShift action_98
action_102 _ = happyReduce_108

action_103 (47) = happyShift action_86
action_103 (50) = happyShift action_127
action_103 (51) = happyShift action_87
action_103 (52) = happyShift action_88
action_103 (54) = happyShift action_89
action_103 (56) = happyShift action_90
action_103 (57) = happyShift action_91
action_103 (58) = happyShift action_92
action_103 (59) = happyShift action_93
action_103 (62) = happyShift action_94
action_103 (63) = happyShift action_95
action_103 (65) = happyShift action_96
action_103 (67) = happyShift action_97
action_103 (68) = happyShift action_98
action_103 _ = happyFail

action_104 (48) = happyShift action_125
action_104 (71) = happyShift action_126
action_104 (25) = happyGoto action_124
action_104 _ = happyFail

action_105 (45) = happyShift action_123
action_105 _ = happyFail

action_106 (41) = happyShift action_106
action_106 (46) = happyShift action_107
action_106 (47) = happyShift action_108
action_106 (50) = happyShift action_109
action_106 (55) = happyShift action_110
action_106 (71) = happyShift action_111
action_106 (7) = happyGoto action_122
action_106 _ = happyReduce_22

action_107 (41) = happyShift action_106
action_107 (46) = happyShift action_107
action_107 (47) = happyShift action_108
action_107 (50) = happyShift action_109
action_107 (55) = happyShift action_110
action_107 (71) = happyShift action_111
action_107 (7) = happyGoto action_121
action_107 _ = happyReduce_22

action_108 (41) = happyShift action_106
action_108 (46) = happyShift action_107
action_108 (47) = happyShift action_108
action_108 (50) = happyShift action_109
action_108 (55) = happyShift action_110
action_108 (71) = happyShift action_111
action_108 (7) = happyGoto action_120
action_108 _ = happyReduce_22

action_109 (41) = happyShift action_106
action_109 (46) = happyShift action_107
action_109 (47) = happyShift action_108
action_109 (50) = happyShift action_109
action_109 (55) = happyShift action_110
action_109 (71) = happyShift action_111
action_109 (7) = happyGoto action_119
action_109 _ = happyReduce_22

action_110 (41) = happyShift action_106
action_110 (46) = happyShift action_107
action_110 (47) = happyShift action_108
action_110 (50) = happyShift action_109
action_110 (55) = happyShift action_110
action_110 (71) = happyShift action_111
action_110 (7) = happyGoto action_118
action_110 _ = happyReduce_22

action_111 (41) = happyShift action_106
action_111 (46) = happyShift action_107
action_111 (47) = happyShift action_108
action_111 (50) = happyShift action_109
action_111 (55) = happyShift action_110
action_111 (71) = happyShift action_111
action_111 (7) = happyGoto action_117
action_111 _ = happyReduce_22

action_112 (46) = happyShift action_115
action_112 (48) = happyShift action_116
action_112 _ = happyFail

action_113 (46) = happyShift action_114
action_113 _ = happyFail

action_114 (71) = happyShift action_214
action_114 (10) = happyGoto action_211
action_114 (11) = happyGoto action_212
action_114 (12) = happyGoto action_213
action_114 _ = happyFail

action_115 (71) = happyShift action_37
action_115 (17) = happyGoto action_210
action_115 _ = happyFail

action_116 (46) = happyShift action_26
action_116 (55) = happyShift action_27
action_116 (57) = happyShift action_28
action_116 (71) = happyShift action_29
action_116 (37) = happyGoto action_209
action_116 _ = happyFail

action_117 _ = happyReduce_23

action_118 _ = happyReduce_25

action_119 _ = happyReduce_27

action_120 _ = happyReduce_24

action_121 _ = happyReduce_26

action_122 _ = happyReduce_28

action_123 _ = happyReduce_13

action_124 (68) = happyShift action_208
action_124 _ = happyFail

action_125 (46) = happyShift action_26
action_125 (55) = happyShift action_27
action_125 (57) = happyShift action_28
action_125 (71) = happyShift action_29
action_125 (37) = happyGoto action_207
action_125 _ = happyFail

action_126 _ = happyReduce_81

action_127 _ = happyReduce_117

action_128 (41) = happyShift action_180
action_128 (50) = happyShift action_206
action_128 _ = happyFail

action_129 (47) = happyShift action_86
action_129 (51) = happyShift action_87
action_129 (52) = happyShift action_88
action_129 (54) = happyShift action_89
action_129 (56) = happyShift action_90
action_129 (57) = happyShift action_91
action_129 (58) = happyShift action_92
action_129 (59) = happyShift action_93
action_129 (62) = happyShift action_94
action_129 (63) = happyShift action_95
action_129 (65) = happyShift action_96
action_129 (67) = happyShift action_97
action_129 (68) = happyShift action_98
action_129 _ = happyReduce_98

action_130 (47) = happyShift action_86
action_130 (51) = happyShift action_87
action_130 (52) = happyShift action_88
action_130 (54) = happyShift action_89
action_130 (56) = happyShift action_90
action_130 (57) = happyShift action_91
action_130 (58) = happyShift action_92
action_130 (59) = happyShift action_93
action_130 (62) = happyShift action_94
action_130 (63) = happyShift action_95
action_130 (65) = happyShift action_96
action_130 (67) = happyShift action_97
action_130 (68) = happyShift action_98
action_130 _ = happyReduce_15

action_131 (51) = happyShift action_87
action_131 (52) = happyShift action_88
action_131 (56) = happyFail
action_131 (57) = happyShift action_91
action_131 (58) = happyShift action_92
action_131 (59) = happyShift action_93
action_131 (62) = happyShift action_94
action_131 (63) = happyShift action_95
action_131 (65) = happyShift action_96
action_131 (67) = happyShift action_97
action_131 (68) = happyFail
action_131 _ = happyReduce_106

action_132 (51) = happyShift action_87
action_132 (52) = happyShift action_88
action_132 (57) = happyShift action_91
action_132 (58) = happyShift action_92
action_132 (59) = happyShift action_93
action_132 (62) = happyFail
action_132 (63) = happyFail
action_132 (65) = happyFail
action_132 (67) = happyFail
action_132 _ = happyReduce_105

action_133 (51) = happyShift action_87
action_133 (52) = happyShift action_88
action_133 (57) = happyShift action_91
action_133 (58) = happyShift action_92
action_133 (59) = happyShift action_93
action_133 (62) = happyFail
action_133 (63) = happyFail
action_133 (65) = happyFail
action_133 (67) = happyFail
action_133 _ = happyReduce_101

action_134 (51) = happyShift action_87
action_134 (52) = happyShift action_88
action_134 (57) = happyShift action_91
action_134 (58) = happyShift action_92
action_134 (59) = happyShift action_93
action_134 (62) = happyFail
action_134 (63) = happyFail
action_134 (65) = happyFail
action_134 (67) = happyFail
action_134 _ = happyReduce_102

action_135 (51) = happyShift action_87
action_135 (52) = happyShift action_88
action_135 (57) = happyShift action_91
action_135 (58) = happyShift action_92
action_135 (59) = happyShift action_93
action_135 (62) = happyFail
action_135 (63) = happyFail
action_135 (65) = happyFail
action_135 (67) = happyFail
action_135 _ = happyReduce_100

action_136 (52) = happyShift action_88
action_136 _ = happyReduce_113

action_137 (52) = happyShift action_88
action_137 _ = happyReduce_114

action_138 (52) = happyShift action_88
action_138 (58) = happyShift action_92
action_138 (59) = happyShift action_93
action_138 _ = happyReduce_111

action_139 (51) = happyShift action_87
action_139 (52) = happyShift action_88
action_139 (56) = happyFail
action_139 (57) = happyShift action_91
action_139 (58) = happyShift action_92
action_139 (59) = happyShift action_93
action_139 (62) = happyShift action_94
action_139 (63) = happyShift action_95
action_139 (65) = happyShift action_96
action_139 (67) = happyShift action_97
action_139 (68) = happyFail
action_139 _ = happyReduce_107

action_140 (51) = happyShift action_87
action_140 (52) = happyShift action_88
action_140 (56) = happyShift action_90
action_140 (57) = happyShift action_91
action_140 (58) = happyShift action_92
action_140 (59) = happyShift action_93
action_140 (62) = happyShift action_94
action_140 (63) = happyShift action_95
action_140 (65) = happyShift action_96
action_140 (67) = happyShift action_97
action_140 (68) = happyShift action_98
action_140 _ = happyReduce_103

action_141 _ = happyReduce_110

action_142 (52) = happyShift action_88
action_142 (58) = happyShift action_92
action_142 (59) = happyShift action_93
action_142 _ = happyReduce_109

action_143 (51) = happyShift action_87
action_143 (52) = happyShift action_88
action_143 (54) = happyShift action_89
action_143 (56) = happyShift action_90
action_143 (57) = happyShift action_91
action_143 (58) = happyShift action_92
action_143 (59) = happyShift action_93
action_143 (62) = happyShift action_94
action_143 (63) = happyShift action_95
action_143 (65) = happyShift action_96
action_143 (67) = happyShift action_97
action_143 (68) = happyShift action_98
action_143 _ = happyReduce_104

action_144 _ = happyReduce_48

action_145 _ = happyReduce_51

action_146 (50) = happyShift action_205
action_146 _ = happyFail

action_147 _ = happyReduce_46

action_148 _ = happyReduce_40

action_149 (71) = happyShift action_204
action_149 _ = happyFail

action_150 _ = happyReduce_30

action_151 (46) = happyShift action_26
action_151 (55) = happyShift action_27
action_151 (57) = happyShift action_28
action_151 (71) = happyShift action_29
action_151 (37) = happyGoto action_203
action_151 _ = happyFail

action_152 _ = happyReduce_32

action_153 _ = happyReduce_43

action_154 (71) = happyShift action_202
action_154 _ = happyFail

action_155 (48) = happyShift action_125
action_155 (71) = happyShift action_126
action_155 (25) = happyGoto action_201
action_155 _ = happyFail

action_156 (71) = happyShift action_75
action_156 (29) = happyGoto action_200
action_156 _ = happyFail

action_157 (68) = happyShift action_199
action_157 _ = happyFail

action_158 (42) = happyShift action_197
action_158 (48) = happyShift action_198
action_158 _ = happyFail

action_159 (45) = happyShift action_195
action_159 (46) = happyShift action_26
action_159 (55) = happyShift action_27
action_159 (57) = happyShift action_28
action_159 (71) = happyShift action_196
action_159 (30) = happyGoto action_192
action_159 (31) = happyGoto action_193
action_159 (36) = happyGoto action_194
action_159 (37) = happyGoto action_129
action_159 _ = happyFail

action_160 (46) = happyShift action_26
action_160 (50) = happyShift action_191
action_160 (55) = happyShift action_27
action_160 (57) = happyShift action_28
action_160 (71) = happyShift action_29
action_160 (36) = happyGoto action_190
action_160 (37) = happyGoto action_129
action_160 _ = happyFail

action_161 (47) = happyShift action_86
action_161 (51) = happyShift action_87
action_161 (52) = happyShift action_88
action_161 (54) = happyShift action_89
action_161 (56) = happyShift action_90
action_161 (57) = happyShift action_91
action_161 (58) = happyShift action_92
action_161 (59) = happyShift action_93
action_161 (62) = happyShift action_94
action_161 (63) = happyShift action_189
action_161 (65) = happyShift action_96
action_161 (67) = happyShift action_97
action_161 (68) = happyShift action_98
action_161 _ = happyFail

action_162 (50) = happyShift action_188
action_162 (53) = happyShift action_166
action_162 _ = happyFail

action_163 (47) = happyShift action_86
action_163 (50) = happyShift action_127
action_163 (51) = happyShift action_87
action_163 (52) = happyShift action_88
action_163 (54) = happyShift action_89
action_163 (56) = happyShift action_90
action_163 (57) = happyShift action_91
action_163 (58) = happyShift action_92
action_163 (59) = happyShift action_93
action_163 (62) = happyShift action_94
action_163 (63) = happyShift action_95
action_163 (65) = happyShift action_96
action_163 (67) = happyShift action_97
action_163 (68) = happyShift action_98
action_163 (69) = happyShift action_165
action_163 _ = happyFail

action_164 (71) = happyShift action_187
action_164 _ = happyFail

action_165 (39) = happyShift action_69
action_165 (46) = happyShift action_70
action_165 (55) = happyShift action_27
action_165 (57) = happyShift action_28
action_165 (62) = happyShift action_71
action_165 (71) = happyShift action_72
action_165 (23) = happyGoto action_186
action_165 (24) = happyGoto action_67
action_165 (37) = happyGoto action_68
action_165 _ = happyFail

action_166 (39) = happyShift action_69
action_166 (46) = happyShift action_70
action_166 (55) = happyShift action_27
action_166 (57) = happyShift action_28
action_166 (62) = happyShift action_71
action_166 (71) = happyShift action_72
action_166 (23) = happyGoto action_185
action_166 (24) = happyGoto action_67
action_166 (37) = happyGoto action_68
action_166 _ = happyFail

action_167 _ = happyReduce_54

action_168 _ = happyReduce_58

action_169 (41) = happyShift action_183
action_169 (50) = happyShift action_184
action_169 _ = happyFail

action_170 _ = happyReduce_92

action_171 (68) = happyShift action_182
action_171 _ = happyFail

action_172 (41) = happyShift action_180
action_172 (45) = happyShift action_181
action_172 _ = happyFail

action_173 _ = happyReduce_63

action_174 (60) = happyShift action_179
action_174 _ = happyFail

action_175 (60) = happyShift action_178
action_175 _ = happyFail

action_176 (60) = happyShift action_177
action_176 _ = happyFail

action_177 (46) = happyShift action_52
action_177 (71) = happyShift action_53
action_177 (72) = happyShift action_54
action_177 (73) = happyShift action_55
action_177 (76) = happyShift action_56
action_177 (20) = happyGoto action_245
action_177 _ = happyFail

action_178 (46) = happyShift action_52
action_178 (71) = happyShift action_53
action_178 (72) = happyShift action_54
action_178 (73) = happyShift action_55
action_178 (76) = happyShift action_56
action_178 (20) = happyGoto action_244
action_178 _ = happyFail

action_179 (46) = happyShift action_52
action_179 (71) = happyShift action_53
action_179 (72) = happyShift action_54
action_179 (73) = happyShift action_55
action_179 (76) = happyShift action_56
action_179 (20) = happyGoto action_243
action_179 _ = happyFail

action_180 (46) = happyShift action_26
action_180 (55) = happyShift action_27
action_180 (57) = happyShift action_28
action_180 (71) = happyShift action_29
action_180 (37) = happyGoto action_242
action_180 _ = happyFail

action_181 _ = happyReduce_64

action_182 (71) = happyShift action_241
action_182 _ = happyFail

action_183 (71) = happyShift action_171
action_183 (33) = happyGoto action_240
action_183 _ = happyFail

action_184 (44) = happyShift action_62
action_184 (22) = happyGoto action_239
action_184 _ = happyReduce_62

action_185 _ = happyReduce_65

action_186 _ = happyReduce_74

action_187 (60) = happyShift action_238
action_187 _ = happyFail

action_188 _ = happyReduce_76

action_189 (42) = happyShift action_237
action_189 (46) = happyShift action_26
action_189 (55) = happyShift action_27
action_189 (57) = happyShift action_28
action_189 (71) = happyShift action_29
action_189 (37) = happyGoto action_134
action_189 _ = happyFail

action_190 (41) = happyShift action_180
action_190 (50) = happyShift action_236
action_190 _ = happyFail

action_191 _ = happyReduce_60

action_192 (41) = happyShift action_234
action_192 (45) = happyShift action_235
action_192 _ = happyFail

action_193 _ = happyReduce_89

action_194 (41) = happyShift action_180
action_194 (45) = happyShift action_233
action_194 _ = happyFail

action_195 _ = happyReduce_80

action_196 (46) = happyShift action_100
action_196 (61) = happyShift action_232
action_196 _ = happyReduce_116

action_197 (39) = happyShift action_69
action_197 (40) = happyShift action_231
action_197 (46) = happyShift action_70
action_197 (55) = happyShift action_27
action_197 (57) = happyShift action_28
action_197 (62) = happyShift action_71
action_197 (71) = happyShift action_72
action_197 (23) = happyGoto action_230
action_197 (24) = happyGoto action_67
action_197 (37) = happyGoto action_68
action_197 _ = happyFail

action_198 (46) = happyShift action_26
action_198 (55) = happyShift action_27
action_198 (57) = happyShift action_28
action_198 (71) = happyShift action_29
action_198 (34) = happyGoto action_227
action_198 (35) = happyGoto action_228
action_198 (37) = happyGoto action_229
action_198 _ = happyFail

action_199 (39) = happyShift action_69
action_199 (46) = happyShift action_70
action_199 (55) = happyShift action_27
action_199 (57) = happyShift action_28
action_199 (62) = happyShift action_71
action_199 (71) = happyShift action_72
action_199 (23) = happyGoto action_226
action_199 (24) = happyGoto action_67
action_199 (37) = happyGoto action_68
action_199 _ = happyFail

action_200 _ = happyReduce_87

action_201 _ = happyReduce_88

action_202 (50) = happyShift action_225
action_202 _ = happyFail

action_203 (47) = happyShift action_86
action_203 (51) = happyShift action_87
action_203 (52) = happyShift action_88
action_203 (54) = happyShift action_89
action_203 (56) = happyShift action_90
action_203 (57) = happyShift action_91
action_203 (58) = happyShift action_92
action_203 (59) = happyShift action_93
action_203 (62) = happyShift action_94
action_203 (63) = happyShift action_95
action_203 (64) = happyShift action_224
action_203 (65) = happyShift action_96
action_203 (67) = happyShift action_97
action_203 (68) = happyShift action_98
action_203 _ = happyFail

action_204 (41) = happyShift action_223
action_204 _ = happyFail

action_205 _ = happyReduce_50

action_206 _ = happyReduce_115

action_207 (47) = happyShift action_86
action_207 (51) = happyShift action_87
action_207 (52) = happyShift action_88
action_207 (54) = happyShift action_89
action_207 (56) = happyShift action_90
action_207 (57) = happyShift action_91
action_207 (58) = happyShift action_92
action_207 (59) = happyShift action_93
action_207 (62) = happyShift action_94
action_207 (63) = happyShift action_95
action_207 (64) = happyShift action_222
action_207 (65) = happyShift action_96
action_207 (67) = happyShift action_97
action_207 (68) = happyShift action_98
action_207 _ = happyFail

action_208 (46) = happyShift action_26
action_208 (55) = happyShift action_27
action_208 (57) = happyShift action_28
action_208 (71) = happyShift action_29
action_208 (37) = happyGoto action_221
action_208 _ = happyFail

action_209 (47) = happyShift action_86
action_209 (51) = happyShift action_87
action_209 (52) = happyShift action_88
action_209 (54) = happyShift action_89
action_209 (56) = happyShift action_90
action_209 (57) = happyShift action_91
action_209 (58) = happyShift action_92
action_209 (59) = happyShift action_93
action_209 (62) = happyShift action_94
action_209 (63) = happyShift action_95
action_209 (64) = happyShift action_220
action_209 (65) = happyShift action_96
action_209 (67) = happyShift action_97
action_209 (68) = happyShift action_98
action_209 _ = happyFail

action_210 (50) = happyShift action_219
action_210 _ = happyFail

action_211 (50) = happyShift action_218
action_211 _ = happyFail

action_212 (41) = happyShift action_217
action_212 _ = happyReduce_34

action_213 (70) = happyShift action_216
action_213 _ = happyFail

action_214 (59) = happyShift action_215
action_214 _ = happyReduce_37

action_215 (71) = happyShift action_214
action_215 (12) = happyGoto action_264
action_215 _ = happyFail

action_216 (71) = happyShift action_263
action_216 _ = happyFail

action_217 (71) = happyShift action_214
action_217 (10) = happyGoto action_262
action_217 (11) = happyGoto action_212
action_217 (12) = happyGoto action_213
action_217 _ = happyFail

action_218 _ = happyReduce_18

action_219 _ = happyReduce_16

action_220 (46) = happyShift action_26
action_220 (55) = happyShift action_27
action_220 (57) = happyShift action_28
action_220 (71) = happyShift action_29
action_220 (37) = happyGoto action_261
action_220 _ = happyFail

action_221 (47) = happyShift action_86
action_221 (51) = happyShift action_87
action_221 (52) = happyShift action_88
action_221 (54) = happyShift action_89
action_221 (56) = happyShift action_90
action_221 (57) = happyShift action_91
action_221 (58) = happyShift action_92
action_221 (59) = happyShift action_93
action_221 (62) = happyShift action_94
action_221 (63) = happyShift action_95
action_221 (65) = happyShift action_96
action_221 (67) = happyShift action_97
action_221 (68) = happyShift action_98
action_221 _ = happyReduce_7

action_222 (46) = happyShift action_26
action_222 (55) = happyShift action_27
action_222 (57) = happyShift action_28
action_222 (71) = happyShift action_29
action_222 (37) = happyGoto action_260
action_222 _ = happyFail

action_223 (71) = happyShift action_259
action_223 _ = happyFail

action_224 (46) = happyShift action_26
action_224 (55) = happyShift action_27
action_224 (57) = happyShift action_28
action_224 (71) = happyShift action_29
action_224 (37) = happyGoto action_258
action_224 _ = happyFail

action_225 _ = happyReduce_44

action_226 (53) = happyShift action_166
action_226 _ = happyReduce_20

action_227 (41) = happyShift action_256
action_227 (49) = happyShift action_257
action_227 _ = happyFail

action_228 _ = happyReduce_95

action_229 (47) = happyShift action_86
action_229 (51) = happyShift action_87
action_229 (52) = happyShift action_88
action_229 (54) = happyShift action_89
action_229 (56) = happyShift action_90
action_229 (57) = happyShift action_91
action_229 (58) = happyShift action_92
action_229 (59) = happyShift action_93
action_229 (61) = happyShift action_255
action_229 (62) = happyShift action_94
action_229 (63) = happyShift action_95
action_229 (65) = happyShift action_96
action_229 (67) = happyShift action_97
action_229 (68) = happyShift action_98
action_229 _ = happyFail

action_230 _ = happyReduce_66

action_231 (46) = happyShift action_254
action_231 _ = happyFail

action_232 (46) = happyShift action_26
action_232 (55) = happyShift action_27
action_232 (57) = happyShift action_28
action_232 (71) = happyShift action_29
action_232 (37) = happyGoto action_253
action_232 _ = happyFail

action_233 _ = happyReduce_78

action_234 (71) = happyShift action_252
action_234 (31) = happyGoto action_251
action_234 _ = happyFail

action_235 _ = happyReduce_79

action_236 (47) = happyReduce_115
action_236 (50) = happyReduce_115
action_236 (51) = happyReduce_115
action_236 (52) = happyReduce_115
action_236 (54) = happyReduce_115
action_236 (56) = happyReduce_115
action_236 (57) = happyReduce_115
action_236 (58) = happyReduce_115
action_236 (59) = happyReduce_115
action_236 (62) = happyReduce_115
action_236 (63) = happyReduce_115
action_236 (65) = happyReduce_115
action_236 (67) = happyReduce_115
action_236 (68) = happyReduce_115
action_236 (69) = happyReduce_115
action_236 (70) = happyReduce_115
action_236 _ = happyReduce_61

action_237 (39) = happyShift action_69
action_237 (46) = happyShift action_70
action_237 (55) = happyShift action_27
action_237 (57) = happyShift action_28
action_237 (62) = happyShift action_71
action_237 (71) = happyShift action_72
action_237 (23) = happyGoto action_250
action_237 (24) = happyGoto action_67
action_237 (37) = happyGoto action_68
action_237 _ = happyFail

action_238 (48) = happyShift action_125
action_238 (71) = happyShift action_126
action_238 (25) = happyGoto action_249
action_238 _ = happyFail

action_239 _ = happyReduce_53

action_240 _ = happyReduce_93

action_241 _ = happyReduce_94

action_242 (47) = happyShift action_86
action_242 (51) = happyShift action_87
action_242 (52) = happyShift action_88
action_242 (54) = happyShift action_89
action_242 (56) = happyShift action_90
action_242 (57) = happyShift action_91
action_242 (58) = happyShift action_92
action_242 (59) = happyShift action_93
action_242 (62) = happyShift action_94
action_242 (63) = happyShift action_95
action_242 (65) = happyShift action_96
action_242 (67) = happyShift action_97
action_242 (68) = happyShift action_98
action_242 _ = happyReduce_99

action_243 (43) = happyShift action_65
action_243 (50) = happyShift action_248
action_243 _ = happyFail

action_244 (43) = happyShift action_65
action_244 (50) = happyShift action_247
action_244 _ = happyFail

action_245 (43) = happyShift action_65
action_245 (50) = happyShift action_246
action_245 _ = happyFail

action_246 _ = happyReduce_56

action_247 _ = happyReduce_57

action_248 _ = happyReduce_55

action_249 (41) = happyShift action_277
action_249 _ = happyFail

action_250 _ = happyReduce_77

action_251 _ = happyReduce_90

action_252 (61) = happyShift action_232
action_252 _ = happyFail

action_253 (47) = happyShift action_86
action_253 (51) = happyShift action_87
action_253 (52) = happyShift action_88
action_253 (54) = happyShift action_89
action_253 (56) = happyShift action_90
action_253 (57) = happyShift action_91
action_253 (58) = happyShift action_92
action_253 (59) = happyShift action_93
action_253 (62) = happyShift action_94
action_253 (63) = happyShift action_95
action_253 (65) = happyShift action_96
action_253 (67) = happyShift action_97
action_253 (68) = happyShift action_98
action_253 _ = happyReduce_91

action_254 (46) = happyShift action_26
action_254 (55) = happyShift action_27
action_254 (57) = happyShift action_28
action_254 (71) = happyShift action_276
action_254 (26) = happyGoto action_272
action_254 (27) = happyGoto action_273
action_254 (37) = happyGoto action_274
action_254 (38) = happyGoto action_275
action_254 _ = happyFail

action_255 (46) = happyShift action_26
action_255 (55) = happyShift action_27
action_255 (57) = happyShift action_28
action_255 (71) = happyShift action_29
action_255 (37) = happyGoto action_271
action_255 _ = happyFail

action_256 (46) = happyShift action_26
action_256 (55) = happyShift action_27
action_256 (57) = happyShift action_28
action_256 (71) = happyShift action_29
action_256 (35) = happyGoto action_270
action_256 (37) = happyGoto action_229
action_256 _ = happyFail

action_257 (42) = happyShift action_269
action_257 _ = happyFail

action_258 (47) = happyShift action_86
action_258 (49) = happyShift action_268
action_258 (51) = happyShift action_87
action_258 (52) = happyShift action_88
action_258 (54) = happyShift action_89
action_258 (56) = happyShift action_90
action_258 (57) = happyShift action_91
action_258 (58) = happyShift action_92
action_258 (59) = happyShift action_93
action_258 (62) = happyShift action_94
action_258 (63) = happyShift action_95
action_258 (65) = happyShift action_96
action_258 (67) = happyShift action_97
action_258 (68) = happyShift action_98
action_258 _ = happyFail

action_259 (50) = happyShift action_267
action_259 _ = happyFail

action_260 (47) = happyShift action_86
action_260 (49) = happyShift action_266
action_260 (51) = happyShift action_87
action_260 (52) = happyShift action_88
action_260 (54) = happyShift action_89
action_260 (56) = happyShift action_90
action_260 (57) = happyShift action_91
action_260 (58) = happyShift action_92
action_260 (59) = happyShift action_93
action_260 (62) = happyShift action_94
action_260 (63) = happyShift action_95
action_260 (65) = happyShift action_96
action_260 (67) = happyShift action_97
action_260 (68) = happyShift action_98
action_260 _ = happyFail

action_261 (47) = happyShift action_86
action_261 (49) = happyShift action_265
action_261 (51) = happyShift action_87
action_261 (52) = happyShift action_88
action_261 (54) = happyShift action_89
action_261 (56) = happyShift action_90
action_261 (57) = happyShift action_91
action_261 (58) = happyShift action_92
action_261 (59) = happyShift action_93
action_261 (62) = happyShift action_94
action_261 (63) = happyShift action_95
action_261 (65) = happyShift action_96
action_261 (67) = happyShift action_97
action_261 (68) = happyShift action_98
action_261 _ = happyFail

action_262 _ = happyReduce_35

action_263 _ = happyReduce_36

action_264 _ = happyReduce_38

action_265 _ = happyReduce_17

action_266 _ = happyReduce_82

action_267 _ = happyReduce_41

action_268 _ = happyReduce_33

action_269 (39) = happyShift action_69
action_269 (40) = happyShift action_285
action_269 (46) = happyShift action_70
action_269 (55) = happyShift action_27
action_269 (57) = happyShift action_28
action_269 (62) = happyShift action_71
action_269 (71) = happyShift action_72
action_269 (23) = happyGoto action_284
action_269 (24) = happyGoto action_67
action_269 (37) = happyGoto action_68
action_269 _ = happyFail

action_270 _ = happyReduce_96

action_271 (47) = happyShift action_86
action_271 (51) = happyShift action_87
action_271 (52) = happyShift action_88
action_271 (54) = happyShift action_89
action_271 (56) = happyShift action_90
action_271 (57) = happyShift action_91
action_271 (58) = happyShift action_92
action_271 (59) = happyShift action_93
action_271 (62) = happyShift action_94
action_271 (63) = happyShift action_95
action_271 (65) = happyShift action_96
action_271 (67) = happyShift action_97
action_271 (68) = happyShift action_98
action_271 _ = happyReduce_97

action_272 (60) = happyShift action_283
action_272 _ = happyFail

action_273 (43) = happyShift action_282
action_273 _ = happyReduce_83

action_274 (47) = happyShift action_86
action_274 (51) = happyShift action_87
action_274 (52) = happyShift action_88
action_274 (54) = happyShift action_89
action_274 (56) = happyShift action_90
action_274 (57) = happyShift action_91
action_274 (58) = happyShift action_92
action_274 (59) = happyShift action_93
action_274 (62) = happyShift action_94
action_274 (63) = happyShift action_95
action_274 (65) = happyShift action_96
action_274 (67) = happyShift action_97
action_274 (68) = happyShift action_98
action_274 (70) = happyShift action_281
action_274 _ = happyFail

action_275 (50) = happyShift action_280
action_275 _ = happyFail

action_276 (46) = happyShift action_100
action_276 (60) = happyShift action_279
action_276 _ = happyReduce_116

action_277 (39) = happyShift action_69
action_277 (46) = happyShift action_70
action_277 (55) = happyShift action_27
action_277 (57) = happyShift action_28
action_277 (62) = happyShift action_71
action_277 (71) = happyShift action_72
action_277 (23) = happyGoto action_278
action_277 (24) = happyGoto action_67
action_277 (37) = happyGoto action_68
action_277 _ = happyFail

action_278 (50) = happyShift action_293
action_278 (53) = happyShift action_166
action_278 _ = happyFail

action_279 (48) = happyShift action_125
action_279 (71) = happyShift action_126
action_279 (25) = happyGoto action_292
action_279 _ = happyFail

action_280 _ = happyReduce_71

action_281 (39) = happyShift action_69
action_281 (46) = happyShift action_70
action_281 (55) = happyShift action_27
action_281 (57) = happyShift action_28
action_281 (62) = happyShift action_71
action_281 (71) = happyShift action_72
action_281 (23) = happyGoto action_291
action_281 (24) = happyGoto action_67
action_281 (37) = happyGoto action_68
action_281 _ = happyFail

action_282 (71) = happyShift action_290
action_282 (26) = happyGoto action_289
action_282 (27) = happyGoto action_273
action_282 _ = happyFail

action_283 (39) = happyShift action_69
action_283 (46) = happyShift action_70
action_283 (48) = happyShift action_288
action_283 (55) = happyShift action_27
action_283 (57) = happyShift action_28
action_283 (62) = happyShift action_71
action_283 (71) = happyShift action_72
action_283 (23) = happyGoto action_287
action_283 (24) = happyGoto action_67
action_283 (37) = happyGoto action_68
action_283 _ = happyFail

action_284 _ = happyReduce_67

action_285 (46) = happyShift action_286
action_285 _ = happyFail

action_286 (46) = happyShift action_26
action_286 (55) = happyShift action_27
action_286 (57) = happyShift action_28
action_286 (71) = happyShift action_276
action_286 (26) = happyGoto action_298
action_286 (27) = happyGoto action_273
action_286 (37) = happyGoto action_274
action_286 (38) = happyGoto action_299
action_286 _ = happyFail

action_287 (50) = happyShift action_297
action_287 (53) = happyShift action_166
action_287 _ = happyFail

action_288 (46) = happyShift action_26
action_288 (55) = happyShift action_27
action_288 (57) = happyShift action_28
action_288 (71) = happyShift action_29
action_288 (34) = happyGoto action_296
action_288 (35) = happyGoto action_228
action_288 (37) = happyGoto action_229
action_288 _ = happyFail

action_289 _ = happyReduce_84

action_290 (60) = happyShift action_279
action_290 _ = happyFail

action_291 (53) = happyShift action_295
action_291 _ = happyReduce_118

action_292 (41) = happyShift action_294
action_292 _ = happyFail

action_293 _ = happyReduce_73

action_294 (46) = happyShift action_26
action_294 (55) = happyShift action_27
action_294 (57) = happyShift action_28
action_294 (71) = happyShift action_29
action_294 (37) = happyGoto action_305
action_294 _ = happyFail

action_295 (39) = happyShift action_69
action_295 (46) = happyShift action_70
action_295 (55) = happyShift action_27
action_295 (57) = happyShift action_28
action_295 (62) = happyShift action_71
action_295 (71) = happyShift action_72
action_295 (23) = happyGoto action_185
action_295 (24) = happyGoto action_67
action_295 (37) = happyGoto action_303
action_295 (38) = happyGoto action_304
action_295 _ = happyFail

action_296 (41) = happyShift action_256
action_296 (49) = happyShift action_302
action_296 _ = happyFail

action_297 _ = happyReduce_68

action_298 (60) = happyShift action_301
action_298 _ = happyFail

action_299 (50) = happyShift action_300
action_299 _ = happyFail

action_300 _ = happyReduce_72

action_301 (39) = happyShift action_69
action_301 (46) = happyShift action_70
action_301 (55) = happyShift action_27
action_301 (57) = happyShift action_28
action_301 (62) = happyShift action_71
action_301 (71) = happyShift action_72
action_301 (23) = happyGoto action_307
action_301 (24) = happyGoto action_67
action_301 (37) = happyGoto action_68
action_301 _ = happyFail

action_302 (39) = happyShift action_69
action_302 (46) = happyShift action_70
action_302 (55) = happyShift action_27
action_302 (57) = happyShift action_28
action_302 (62) = happyShift action_71
action_302 (71) = happyShift action_72
action_302 (23) = happyGoto action_306
action_302 (24) = happyGoto action_67
action_302 (37) = happyGoto action_68
action_302 _ = happyFail

action_303 (47) = happyShift action_86
action_303 (51) = happyShift action_87
action_303 (52) = happyShift action_88
action_303 (54) = happyShift action_89
action_303 (56) = happyShift action_90
action_303 (57) = happyShift action_91
action_303 (58) = happyShift action_92
action_303 (59) = happyShift action_93
action_303 (62) = happyShift action_94
action_303 (63) = happyShift action_95
action_303 (65) = happyShift action_96
action_303 (67) = happyShift action_97
action_303 (68) = happyShift action_98
action_303 (69) = happyShift action_165
action_303 (70) = happyShift action_281
action_303 _ = happyFail

action_304 _ = happyReduce_119

action_305 (47) = happyShift action_86
action_305 (51) = happyShift action_87
action_305 (52) = happyShift action_88
action_305 (54) = happyShift action_89
action_305 (56) = happyShift action_90
action_305 (57) = happyShift action_91
action_305 (58) = happyShift action_92
action_305 (59) = happyShift action_93
action_305 (62) = happyShift action_94
action_305 (63) = happyShift action_95
action_305 (65) = happyShift action_96
action_305 (67) = happyShift action_97
action_305 (68) = happyShift action_98
action_305 _ = happyReduce_85

action_306 (50) = happyShift action_309
action_306 (53) = happyShift action_166
action_306 _ = happyFail

action_307 (50) = happyShift action_308
action_307 (53) = happyShift action_166
action_307 _ = happyFail

action_308 _ = happyReduce_69

action_309 _ = happyReduce_70

happyReduce_1 = happySpecReduce_1  4 happyReduction_1
happyReduction_1 (HappyAbsSyn5  happy_var_1)
	 =  HappyAbsSyn4
		 (happy_var_1
	)
happyReduction_1 _  = notHappyAtAll 

happyReduce_2 = happySpecReduce_0  5 happyReduction_2
happyReduction_2  =  HappyAbsSyn5
		 ([]
	)

happyReduce_3 = happySpecReduce_2  5 happyReduction_3
happyReduction_3 (HappyAbsSyn5  happy_var_2)
	(HappyAbsSyn6  happy_var_1)
	 =  HappyAbsSyn5
		 (happy_var_1 : happy_var_2
	)
happyReduction_3 _ _  = notHappyAtAll 

happyReduce_4 = happySpecReduce_2  6 happyReduction_4
happyReduction_4 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataHiding happy_var_2
	)
happyReduction_4 _ _  = notHappyAtAll 

happyReduce_5 = happySpecReduce_2  6 happyReduction_5
happyReduction_5 (HappyAbsSyn15  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataRenaming happy_var_2
	)
happyReduction_5 _ _  = notHappyAtAll 

happyReduce_6 = happySpecReduce_2  6 happyReduction_6
happyReduction_6 (HappyAbsSyn8  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataActions happy_var_2
	)
happyReduction_6 _ _  = notHappyAtAll 

happyReduce_7 = happyReduce 6 6 happyReduction_7
happyReduction_7 ((HappyAbsSyn37  happy_var_6) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn25  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataGlobal (happy_var_2, happy_var_4) happy_var_6
	) `HappyStk` happyRest

happyReduce_8 = happySpecReduce_2  6 happyReduction_8
happyReduction_8 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataEncapsulation happy_var_2
	)
happyReduction_8 _ _  = notHappyAtAll 

happyReduce_9 = happySpecReduce_2  6 happyReduction_9
happyReduction_9 (HappyAbsSyn13  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataCommunication happy_var_2
	)
happyReduction_9 _ _  = notHappyAtAll 

happyReduce_10 = happySpecReduce_2  6 happyReduction_10
happyReduction_10 (HappyAbsSyn17  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataNoCommunication happy_var_2
	)
happyReduction_10 _ _  = notHappyAtAll 

happyReduce_11 = happySpecReduce_2  6 happyReduction_11
happyReduction_11 (HappyAbsSyn18  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataReach happy_var_2
	)
happyReduction_11 _ _  = notHappyAtAll 

happyReduce_12 = happySpecReduce_2  6 happyReduction_12
happyReduction_12 (HappyAbsSyn37  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataReachCondition happy_var_2
	)
happyReduction_12 _ _  = notHappyAtAll 

happyReduce_13 = happyReduce 4 6 happyReduction_13
happyReduction_13 (_ `HappyStk`
	(HappyAbsSyn7  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataUntilFormula happy_var_3
	) `HappyStk` happyRest

happyReduce_14 = happySpecReduce_2  6 happyReduction_14
happyReduction_14 (HappyAbsSyn37  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (DataFormula happy_var_2
	)
happyReduction_14 _ _  = notHappyAtAll 

happyReduce_15 = happyReduce 4 6 happyReduction_15
happyReduction_15 ((HappyAbsSyn37  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataConstant happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_16 = happyReduce 6 6 happyReduction_16
happyReduction_16 (_ `HappyStk`
	(HappyAbsSyn17  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataEnumType happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_17 = happyReduce 8 6 happyReduction_17
happyReduction_17 (_ `HappyStk`
	(HappyAbsSyn37  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn37  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataRangeType happy_var_2 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_18 = happyReduce 6 6 happyReduction_18
happyReduction_18 (_ `HappyStk`
	(HappyAbsSyn10  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (DataFunction happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_19 = happySpecReduce_2  6 happyReduction_19
happyReduction_19 (HappyAbsSyn20  happy_var_2)
	_
	 =  HappyAbsSyn6
		 (ParserInitialProcess happy_var_2
	)
happyReduction_19 _ _  = notHappyAtAll 

happyReduce_20 = happyReduce 6 6 happyReduction_20
happyReduction_20 ((HappyAbsSyn23  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn28  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 (reverse happy_var_3) happy_var_6)
	) `HappyStk` happyRest

happyReduce_21 = happySpecReduce_3  6 happyReduction_21
happyReduction_21 (HappyAbsSyn23  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn6
		 (ParserProcess (Process happy_var_1 [] happy_var_3)
	)
happyReduction_21 _ _ _  = notHappyAtAll 

happyReduce_22 = happySpecReduce_0  7 happyReduction_22
happyReduction_22  =  HappyAbsSyn7
		 (""
	)

happyReduce_23 = happySpecReduce_2  7 happyReduction_23
happyReduction_23 (HappyAbsSyn7  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn7
		 (happy_var_1 ++ " " ++ happy_var_2
	)
happyReduction_23 _ _  = notHappyAtAll 

happyReduce_24 = happySpecReduce_2  7 happyReduction_24
happyReduction_24 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("|" ++ " " ++ happy_var_2
	)
happyReduction_24 _ _  = notHappyAtAll 

happyReduce_25 = happySpecReduce_2  7 happyReduction_25
happyReduction_25 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("!" ++ " " ++ happy_var_2
	)
happyReduction_25 _ _  = notHappyAtAll 

happyReduce_26 = happySpecReduce_2  7 happyReduction_26
happyReduction_26 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("(" ++ " " ++ happy_var_2
	)
happyReduction_26 _ _  = notHappyAtAll 

happyReduce_27 = happySpecReduce_2  7 happyReduction_27
happyReduction_27 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 (")" ++ " " ++ happy_var_2
	)
happyReduction_27 _ _  = notHappyAtAll 

happyReduce_28 = happySpecReduce_2  7 happyReduction_28
happyReduction_28 (HappyAbsSyn7  happy_var_2)
	_
	 =  HappyAbsSyn7
		 ("," ++ " " ++ happy_var_2
	)
happyReduction_28 _ _  = notHappyAtAll 

happyReduce_29 = happySpecReduce_1  8 happyReduction_29
happyReduction_29 (HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 ([happy_var_1]
	)
happyReduction_29 _  = notHappyAtAll 

happyReduce_30 = happySpecReduce_3  8 happyReduction_30
happyReduction_30 (HappyAbsSyn8  happy_var_3)
	_
	(HappyAbsSyn9  happy_var_1)
	 =  HappyAbsSyn8
		 (happy_var_1:happy_var_3
	)
happyReduction_30 _ _ _  = notHappyAtAll 

happyReduce_31 = happySpecReduce_1  9 happyReduction_31
happyReduction_31 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, None)
	)
happyReduction_31 _  = notHappyAtAll 

happyReduce_32 = happySpecReduce_3  9 happyReduction_32
happyReduction_32 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn9
		 ((happy_var_1, Bool)
	)
happyReduction_32 _ _ _  = notHappyAtAll 

happyReduce_33 = happyReduce 7 9 happyReduction_33
happyReduction_33 (_ `HappyStk`
	(HappyAbsSyn37  happy_var_6) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn37  happy_var_4) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn9
		 ((happy_var_1, IntRange happy_var_4 happy_var_6)
	) `HappyStk` happyRest

happyReduce_34 = happySpecReduce_1  10 happyReduction_34
happyReduction_34 (HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 ([happy_var_1]
	)
happyReduction_34 _  = notHappyAtAll 

happyReduce_35 = happySpecReduce_3  10 happyReduction_35
happyReduction_35 (HappyAbsSyn10  happy_var_3)
	_
	(HappyAbsSyn11  happy_var_1)
	 =  HappyAbsSyn10
		 (happy_var_1:happy_var_3
	)
happyReduction_35 _ _ _  = notHappyAtAll 

happyReduce_36 = happySpecReduce_3  11 happyReduction_36
happyReduction_36 (HappyTerminal (TokenString happy_var_3))
	_
	(HappyAbsSyn12  happy_var_1)
	 =  HappyAbsSyn11
		 ((happy_var_1, happy_var_3)
	)
happyReduction_36 _ _ _  = notHappyAtAll 

happyReduce_37 = happySpecReduce_1  12 happyReduction_37
happyReduction_37 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 ([happy_var_1]
	)
happyReduction_37 _  = notHappyAtAll 

happyReduce_38 = happySpecReduce_3  12 happyReduction_38
happyReduction_38 (HappyAbsSyn12  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn12
		 (happy_var_1:happy_var_3
	)
happyReduction_38 _ _ _  = notHappyAtAll 

happyReduce_39 = happySpecReduce_1  13 happyReduction_39
happyReduction_39 (HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 ([happy_var_1]
	)
happyReduction_39 _  = notHappyAtAll 

happyReduce_40 = happySpecReduce_3  13 happyReduction_40
happyReduction_40 (HappyAbsSyn13  happy_var_3)
	_
	(HappyAbsSyn14  happy_var_1)
	 =  HappyAbsSyn13
		 (happy_var_1 : happy_var_3
	)
happyReduction_40 _ _ _  = notHappyAtAll 

happyReduce_41 = happyReduce 7 14 happyReduction_41
happyReduction_41 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_6)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn14
		 ((happy_var_2,happy_var_4,happy_var_6)
	) `HappyStk` happyRest

happyReduce_42 = happySpecReduce_1  15 happyReduction_42
happyReduction_42 (HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 ([happy_var_1]
	)
happyReduction_42 _  = notHappyAtAll 

happyReduce_43 = happySpecReduce_3  15 happyReduction_43
happyReduction_43 (HappyAbsSyn15  happy_var_3)
	_
	(HappyAbsSyn16  happy_var_1)
	 =  HappyAbsSyn15
		 (happy_var_1 : happy_var_3
	)
happyReduction_43 _ _ _  = notHappyAtAll 

happyReduce_44 = happyReduce 5 16 happyReduction_44
happyReduction_44 (_ `HappyStk`
	(HappyTerminal (TokenString happy_var_4)) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_2)) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn16
		 ((happy_var_2,happy_var_4)
	) `HappyStk` happyRest

happyReduce_45 = happySpecReduce_1  17 happyReduction_45
happyReduction_45 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 ([happy_var_1]
	)
happyReduction_45 _  = notHappyAtAll 

happyReduce_46 = happySpecReduce_3  17 happyReduction_46
happyReduction_46 (HappyAbsSyn17  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn17
		 (happy_var_1 : happy_var_3
	)
happyReduction_46 _ _ _  = notHappyAtAll 

happyReduce_47 = happySpecReduce_1  18 happyReduction_47
happyReduction_47 (HappyAbsSyn19  happy_var_1)
	 =  HappyAbsSyn18
		 ([happy_var_1]
	)
happyReduction_47 _  = notHappyAtAll 

happyReduce_48 = happySpecReduce_3  18 happyReduction_48
happyReduction_48 (HappyAbsSyn18  happy_var_3)
	_
	(HappyAbsSyn19  happy_var_1)
	 =  HappyAbsSyn18
		 (happy_var_1 : happy_var_3
	)
happyReduction_48 _ _ _  = notHappyAtAll 

happyReduce_49 = happySpecReduce_1  19 happyReduction_49
happyReduction_49 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn19
		 (happy_var_1
	)
happyReduction_49 _  = notHappyAtAll 

happyReduce_50 = happyReduce 4 19 happyReduction_50
happyReduction_50 (_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn19
		 (happy_var_1 ++ commaList happy_var_3
	) `HappyStk` happyRest

happyReduce_51 = happySpecReduce_3  19 happyReduction_51
happyReduction_51 _
	(HappyAbsSyn19  happy_var_2)
	_
	 =  HappyAbsSyn19
		 ("rate(" ++ happy_var_2 ++ ")"
	)
happyReduction_51 _ _ _  = notHappyAtAll 

happyReduce_52 = happySpecReduce_2  20 happyReduction_52
happyReduction_52 (HappyAbsSyn22  happy_var_2)
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn20
		 (InitSingleProcess (InitialProcess happy_var_1 happy_var_2 [])
	)
happyReduction_52 _ _  = notHappyAtAll 

happyReduce_53 = happyReduce 5 20 happyReduction_53
happyReduction_53 ((HappyAbsSyn22  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn32  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn20
		 (InitSingleProcess (InitialProcess happy_var_1 happy_var_5 happy_var_3)
	) `HappyStk` happyRest

happyReduce_54 = happySpecReduce_3  20 happyReduction_54
happyReduction_54 (HappyAbsSyn20  happy_var_3)
	_
	(HappyAbsSyn20  happy_var_1)
	 =  HappyAbsSyn20
		 (InitParallel happy_var_1 happy_var_3
	)
happyReduction_54 _ _ _  = notHappyAtAll 

happyReduce_55 = happyReduce 6 20 happyReduction_55
happyReduction_55 (_ `HappyStk`
	(HappyAbsSyn20  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn20
		 (InitHiding happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_56 = happyReduce 6 20 happyReduction_56
happyReduction_56 (_ `HappyStk`
	(HappyAbsSyn20  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn17  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn20
		 (InitEncapsulation happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_57 = happyReduce 6 20 happyReduction_57
happyReduction_57 (_ `HappyStk`
	(HappyAbsSyn20  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn15  happy_var_3) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn20
		 (InitRenaming happy_var_3 happy_var_5
	) `HappyStk` happyRest

happyReduce_58 = happySpecReduce_3  20 happyReduction_58
happyReduction_58 _
	(HappyAbsSyn20  happy_var_2)
	_
	 =  HappyAbsSyn20
		 (happy_var_2
	)
happyReduction_58 _ _ _  = notHappyAtAll 

happyReduce_59 = happySpecReduce_0  21 happyReduction_59
happyReduction_59  =  HappyAbsSyn21
		 ([]
	)

happyReduce_60 = happySpecReduce_2  21 happyReduction_60
happyReduction_60 _
	_
	 =  HappyAbsSyn21
		 ([]
	)

happyReduce_61 = happySpecReduce_3  21 happyReduction_61
happyReduction_61 _
	(HappyAbsSyn36  happy_var_2)
	_
	 =  HappyAbsSyn21
		 (reverse happy_var_2
	)
happyReduction_61 _ _ _  = notHappyAtAll 

happyReduce_62 = happySpecReduce_0  22 happyReduction_62
happyReduction_62  =  HappyAbsSyn22
		 ([]
	)

happyReduce_63 = happySpecReduce_2  22 happyReduction_63
happyReduction_63 _
	_
	 =  HappyAbsSyn22
		 ([]
	)

happyReduce_64 = happySpecReduce_3  22 happyReduction_64
happyReduction_64 _
	(HappyAbsSyn36  happy_var_2)
	_
	 =  HappyAbsSyn22
		 (reverse happy_var_2
	)
happyReduction_64 _ _ _  = notHappyAtAll 

happyReduce_65 = happySpecReduce_3  23 happyReduction_65
happyReduction_65 (HappyAbsSyn23  happy_var_3)
	_
	(HappyAbsSyn23  happy_var_1)
	 =  HappyAbsSyn23
		 (Plus ([happy_var_1] ++ [happy_var_3])
	)
happyReduction_65 _ _ _  = notHappyAtAll 

happyReduce_66 = happyReduce 4 23 happyReduction_66
happyReduction_66 ((HappyAbsSyn23  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix happy_var_1 happy_var_2 [("i0", TypeRange 1 1, (Variable "1"))] happy_var_4
	) `HappyStk` happyRest

happyReduce_67 = happyReduce 7 23 happyReduction_67
happyReduction_67 ((HappyAbsSyn23  happy_var_7) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn34  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix (happy_var_1  ++ "{" ++ (printExpressions happy_var_4) ++ "}") happy_var_2 [("i0", TypeRange 1 1, (Variable "1"))] happy_var_7
	) `HappyStk` happyRest

happyReduce_68 = happyReduce 9 23 happyReduction_68
happyReduction_68 (_ `HappyStk`
	(HappyAbsSyn23  happy_var_8) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn26  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix happy_var_1 happy_var_2 happy_var_6 happy_var_8
	) `HappyStk` happyRest

happyReduce_69 = happyReduce 12 23 happyReduction_69
happyReduction_69 (_ `HappyStk`
	(HappyAbsSyn23  happy_var_11) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn26  happy_var_9) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn34  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix (happy_var_1  ++ "{" ++ (printExpressions happy_var_4) ++ "}") happy_var_2 happy_var_9 happy_var_11
	) `HappyStk` happyRest

happyReduce_70 = happyReduce 12 23 happyReduction_70
happyReduction_70 (_ `HappyStk`
	(HappyAbsSyn23  happy_var_11) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn34  happy_var_9) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn26  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix (happy_var_1  ++ "{" ++ (printExpressions happy_var_9) ++ "}") happy_var_2 happy_var_6 happy_var_11
	) `HappyStk` happyRest

happyReduce_71 = happyReduce 7 23 happyReduction_71
happyReduction_71 (_ `HappyStk`
	(HappyAbsSyn38  happy_var_6) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix2 happy_var_1 happy_var_2 happy_var_6
	) `HappyStk` happyRest

happyReduce_72 = happyReduce 10 23 happyReduction_72
happyReduction_72 (_ `HappyStk`
	(HappyAbsSyn38  happy_var_9) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn34  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn21  happy_var_2) `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (ActionPrefix2 (happy_var_1  ++ "{" ++ (printExpressions happy_var_4) ++ "}") happy_var_2 happy_var_9
	) `HappyStk` happyRest

happyReduce_73 = happyReduce 8 23 happyReduction_73
happyReduction_73 (_ `HappyStk`
	(HappyAbsSyn23  happy_var_7) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn25  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_3)) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (Sum happy_var_3 happy_var_5 happy_var_7
	) `HappyStk` happyRest

happyReduce_74 = happySpecReduce_3  23 happyReduction_74
happyReduction_74 (HappyAbsSyn23  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn23
		 (Implication happy_var_1 happy_var_3
	)
happyReduction_74 _ _ _  = notHappyAtAll 

happyReduce_75 = happySpecReduce_1  23 happyReduction_75
happyReduction_75 (HappyAbsSyn24  happy_var_1)
	 =  HappyAbsSyn23
		 (happy_var_1
	)
happyReduction_75 _  = notHappyAtAll 

happyReduce_76 = happySpecReduce_3  23 happyReduction_76
happyReduction_76 _
	(HappyAbsSyn23  happy_var_2)
	_
	 =  HappyAbsSyn23
		 (happy_var_2
	)
happyReduction_76 _ _ _  = notHappyAtAll 

happyReduce_77 = happyReduce 5 23 happyReduction_77
happyReduction_77 ((HappyAbsSyn23  happy_var_5) `HappyStk`
	_ `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn37  happy_var_2) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn23
		 (LambdaPrefix happy_var_2 happy_var_5
	) `HappyStk` happyRest

happyReduce_78 = happyReduce 4 24 happyReduction_78
happyReduction_78 (_ `HappyStk`
	(HappyAbsSyn36  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn24
		 (ProcessInstance happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_79 = happyReduce 4 24 happyReduction_79
happyReduction_79 (_ `HappyStk`
	(HappyAbsSyn30  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn24
		 (ProcessInstance2 happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_80 = happySpecReduce_3  24 happyReduction_80
happyReduction_80 _
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn24
		 (ProcessInstance2 happy_var_1 []
	)
happyReduction_80 _ _ _  = notHappyAtAll 

happyReduce_81 = happySpecReduce_1  25 happyReduction_81
happyReduction_81 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn25
		 (TypeName happy_var_1
	)
happyReduction_81 _  = notHappyAtAll 

happyReduce_82 = happyReduce 5 25 happyReduction_82
happyReduction_82 (_ `HappyStk`
	(HappyAbsSyn37  happy_var_4) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn37  happy_var_2) `HappyStk`
	_ `HappyStk`
	happyRest)
	 = HappyAbsSyn25
		 (TypeRangeExpressions happy_var_2 happy_var_4
	) `HappyStk` happyRest

happyReduce_83 = happySpecReduce_1  26 happyReduction_83
happyReduction_83 (HappyAbsSyn27  happy_var_1)
	 =  HappyAbsSyn26
		 ([happy_var_1]
	)
happyReduction_83 _  = notHappyAtAll 

happyReduce_84 = happySpecReduce_3  26 happyReduction_84
happyReduction_84 (HappyAbsSyn26  happy_var_3)
	_
	(HappyAbsSyn27  happy_var_1)
	 =  HappyAbsSyn26
		 (happy_var_1 : happy_var_3
	)
happyReduction_84 _ _ _  = notHappyAtAll 

happyReduce_85 = happyReduce 5 27 happyReduction_85
happyReduction_85 ((HappyAbsSyn37  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn25  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn27
		 ((happy_var_1, happy_var_3, happy_var_5)
	) `HappyStk` happyRest

happyReduce_86 = happySpecReduce_1  28 happyReduction_86
happyReduction_86 (HappyAbsSyn29  happy_var_1)
	 =  HappyAbsSyn28
		 ([happy_var_1]
	)
happyReduction_86 _  = notHappyAtAll 

happyReduce_87 = happySpecReduce_3  28 happyReduction_87
happyReduction_87 (HappyAbsSyn29  happy_var_3)
	_
	(HappyAbsSyn28  happy_var_1)
	 =  HappyAbsSyn28
		 (happy_var_3 : happy_var_1
	)
happyReduction_87 _ _ _  = notHappyAtAll 

happyReduce_88 = happySpecReduce_3  29 happyReduction_88
happyReduction_88 (HappyAbsSyn25  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn29
		 ((happy_var_1, happy_var_3)
	)
happyReduction_88 _ _ _  = notHappyAtAll 

happyReduce_89 = happySpecReduce_1  30 happyReduction_89
happyReduction_89 (HappyAbsSyn31  happy_var_1)
	 =  HappyAbsSyn30
		 ([happy_var_1]
	)
happyReduction_89 _  = notHappyAtAll 

happyReduce_90 = happySpecReduce_3  30 happyReduction_90
happyReduction_90 (HappyAbsSyn31  happy_var_3)
	_
	(HappyAbsSyn30  happy_var_1)
	 =  HappyAbsSyn30
		 (happy_var_3 : happy_var_1
	)
happyReduction_90 _ _ _  = notHappyAtAll 

happyReduce_91 = happySpecReduce_3  31 happyReduction_91
happyReduction_91 (HappyAbsSyn37  happy_var_3)
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn31
		 ((happy_var_1, happy_var_3)
	)
happyReduction_91 _ _ _  = notHappyAtAll 

happyReduce_92 = happySpecReduce_1  32 happyReduction_92
happyReduction_92 (HappyAbsSyn33  happy_var_1)
	 =  HappyAbsSyn32
		 ([happy_var_1]
	)
happyReduction_92 _  = notHappyAtAll 

happyReduce_93 = happySpecReduce_3  32 happyReduction_93
happyReduction_93 (HappyAbsSyn33  happy_var_3)
	_
	(HappyAbsSyn32  happy_var_1)
	 =  HappyAbsSyn32
		 (happy_var_3 : happy_var_1
	)
happyReduction_93 _ _ _  = notHappyAtAll 

happyReduce_94 = happySpecReduce_3  33 happyReduction_94
happyReduction_94 (HappyTerminal (TokenString happy_var_3))
	_
	(HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn33
		 ((happy_var_1, happy_var_3)
	)
happyReduction_94 _ _ _  = notHappyAtAll 

happyReduce_95 = happySpecReduce_1  34 happyReduction_95
happyReduction_95 (HappyAbsSyn35  happy_var_1)
	 =  HappyAbsSyn34
		 ([happy_var_1]
	)
happyReduction_95 _  = notHappyAtAll 

happyReduce_96 = happySpecReduce_3  34 happyReduction_96
happyReduction_96 (HappyAbsSyn35  happy_var_3)
	_
	(HappyAbsSyn34  happy_var_1)
	 =  HappyAbsSyn34
		 (happy_var_3 : happy_var_1
	)
happyReduction_96 _ _ _  = notHappyAtAll 

happyReduce_97 = happySpecReduce_3  35 happyReduction_97
happyReduction_97 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn35
		 ((happy_var_1, happy_var_3)
	)
happyReduction_97 _ _ _  = notHappyAtAll 

happyReduce_98 = happySpecReduce_1  36 happyReduction_98
happyReduction_98 (HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn36
		 ([happy_var_1]
	)
happyReduction_98 _  = notHappyAtAll 

happyReduce_99 = happySpecReduce_3  36 happyReduction_99
happyReduction_99 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn36  happy_var_1)
	 =  HappyAbsSyn36
		 (happy_var_3 : happy_var_1
	)
happyReduction_99 _ _ _  = notHappyAtAll 

happyReduce_100 = happySpecReduce_3  37 happyReduction_100
happyReduction_100 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "lt" [happy_var_1, happy_var_3]
	)
happyReduction_100 _ _ _  = notHappyAtAll 

happyReduce_101 = happySpecReduce_3  37 happyReduction_101
happyReduction_101 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "leq" [happy_var_1, happy_var_3]
	)
happyReduction_101 _ _ _  = notHappyAtAll 

happyReduce_102 = happySpecReduce_3  37 happyReduction_102
happyReduction_102 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "gt" [happy_var_1, happy_var_3]
	)
happyReduction_102 _ _ _  = notHappyAtAll 

happyReduce_103 = happySpecReduce_3  37 happyReduction_103
happyReduction_103 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "and" [happy_var_1, happy_var_3]
	)
happyReduction_103 _ _ _  = notHappyAtAll 

happyReduce_104 = happySpecReduce_3  37 happyReduction_104
happyReduction_104 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "or" [happy_var_1, happy_var_3]
	)
happyReduction_104 _ _ _  = notHappyAtAll 

happyReduce_105 = happySpecReduce_3  37 happyReduction_105
happyReduction_105 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "geq" [happy_var_1, happy_var_3]
	)
happyReduction_105 _ _ _  = notHappyAtAll 

happyReduce_106 = happySpecReduce_3  37 happyReduction_106
happyReduction_106 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "eq" [happy_var_1, happy_var_3]
	)
happyReduction_106 _ _ _  = notHappyAtAll 

happyReduce_107 = happySpecReduce_3  37 happyReduction_107
happyReduction_107 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "not" [Function "eq" [happy_var_1, happy_var_3]]
	)
happyReduction_107 _ _ _  = notHappyAtAll 

happyReduce_108 = happySpecReduce_2  37 happyReduction_108
happyReduction_108 (HappyAbsSyn37  happy_var_2)
	_
	 =  HappyAbsSyn37
		 (Function "not" [happy_var_2]
	)
happyReduction_108 _ _  = notHappyAtAll 

happyReduce_109 = happySpecReduce_3  37 happyReduction_109
happyReduction_109 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "plus" [happy_var_1, happy_var_3]
	)
happyReduction_109 _ _ _  = notHappyAtAll 

happyReduce_110 = happySpecReduce_3  37 happyReduction_110
happyReduction_110 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "power" [happy_var_1, happy_var_3]
	)
happyReduction_110 _ _ _  = notHappyAtAll 

happyReduce_111 = happySpecReduce_3  37 happyReduction_111
happyReduction_111 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "minus" [happy_var_1, happy_var_3]
	)
happyReduction_111 _ _ _  = notHappyAtAll 

happyReduce_112 = happySpecReduce_2  37 happyReduction_112
happyReduction_112 (HappyAbsSyn37  happy_var_2)
	_
	 =  HappyAbsSyn37
		 (Function "multiply" [Variable "-1", happy_var_2]
	)
happyReduction_112 _ _  = notHappyAtAll 

happyReduce_113 = happySpecReduce_3  37 happyReduction_113
happyReduction_113 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "multiply" [happy_var_1, happy_var_3]
	)
happyReduction_113 _ _ _  = notHappyAtAll 

happyReduce_114 = happySpecReduce_3  37 happyReduction_114
happyReduction_114 (HappyAbsSyn37  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn37
		 (Function "divide" [happy_var_1, happy_var_3]
	)
happyReduction_114 _ _ _  = notHappyAtAll 

happyReduce_115 = happyReduce 4 37 happyReduction_115
happyReduction_115 (_ `HappyStk`
	(HappyAbsSyn36  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyTerminal (TokenString happy_var_1)) `HappyStk`
	happyRest)
	 = HappyAbsSyn37
		 (Function happy_var_1 (reverse happy_var_3)
	) `HappyStk` happyRest

happyReduce_116 = happySpecReduce_1  37 happyReduction_116
happyReduction_116 (HappyTerminal (TokenString happy_var_1))
	 =  HappyAbsSyn37
		 (Variable happy_var_1
	)
happyReduction_116 _  = notHappyAtAll 

happyReduce_117 = happySpecReduce_3  37 happyReduction_117
happyReduction_117 _
	(HappyAbsSyn37  happy_var_2)
	_
	 =  HappyAbsSyn37
		 (happy_var_2
	)
happyReduction_117 _ _ _  = notHappyAtAll 

happyReduce_118 = happySpecReduce_3  38 happyReduction_118
happyReduction_118 (HappyAbsSyn23  happy_var_3)
	_
	(HappyAbsSyn37  happy_var_1)
	 =  HappyAbsSyn38
		 ([(happy_var_1, happy_var_3)]
	)
happyReduction_118 _ _ _  = notHappyAtAll 

happyReduce_119 = happyReduce 5 38 happyReduction_119
happyReduction_119 ((HappyAbsSyn38  happy_var_5) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn23  happy_var_3) `HappyStk`
	_ `HappyStk`
	(HappyAbsSyn37  happy_var_1) `HappyStk`
	happyRest)
	 = HappyAbsSyn38
		 ((happy_var_1, happy_var_3) : happy_var_5
	) `HappyStk` happyRest

happyNewToken action sts stk
	= lexer(\tk -> 
	let cont i = action i i tk (HappyState action) sts stk in
	case tk of {
	TokenEOF -> action 89 89 tk (HappyState action) sts stk;
	TokenSum -> cont 39;
	TokenPSum -> cont 40;
	TokenComma -> cont 41;
	TokenSeq -> cont 42;
	TokenIndependence -> cont 43;
	TokenOSB -> cont 44;
	TokenCSB -> cont 45;
	TokenOB -> cont 46;
	TokenOr -> cont 47;
	TokenOA -> cont 48;
	TokenCA -> cont 49;
	TokenCB -> cont 50;
	TokenPlus -> cont 51;
	TokenPower -> cont 52;
	TokenPlusPlus -> cont 53;
	TokenAnd -> cont 54;
	TokenNot -> cont 55;
	TokenNotEqual -> cont 56;
	TokenMinus -> cont 57;
	TokenDivide -> cont 58;
	TokenMultiply -> cont 59;
	TokenColon -> cont 60;
	TokenBecomes -> cont 61;
	TokenSmaller -> cont 62;
	TokenGreater -> cont 63;
	TokenDotDot -> cont 64;
	TokenSmallerEq -> cont 65;
	TokenInit -> cont 66;
	TokenGreaterEq -> cont 67;
	TokenEqual -> cont 68;
	TokenImplies -> cont 69;
	TokenProbDef -> cont 70;
	TokenString happy_dollar_dollar -> cont 71;
	TokenHide -> cont 72;
	TokenRename -> cont 73;
	TokenBool -> cont 74;
	TokenActions -> cont 75;
	TokenEncap -> cont 76;
	TokenComm -> cont 77;
	TokenNoComm -> cont 78;
	TokenReach -> cont 79;
	TokenReachCondition -> cont 80;
	TokenConstant -> cont 81;
	TokenFormula -> cont 82;
	TokenGlobal -> cont 83;
	TokenUntilFormula -> cont 84;
	TokenType -> cont 85;
	TokenFunction -> cont 86;
	TokenNat -> cont 87;
	TokenQueue -> cont 88;
	_ -> happyError' tk
	})

happyError_ 89 tk = happyError' tk
happyError_ _ tk = happyError' tk

happyThen :: () => ParserMonad a -> (a -> ParserMonad b) -> ParserMonad b
happyThen = (thenParserMonad)
happyReturn :: () => a -> ParserMonad a
happyReturn = (returnParserMonad)
happyThen1 = happyThen
happyReturn1 :: () => a -> ParserMonad a
happyReturn1 = happyReturn
happyError' :: () => (Token) -> ParserMonad a
happyError' tk = parseError tk

parse = happySomeParser where
  happySomeParser = happyThen (happyParse action_0) (\x -> case x of {HappyAbsSyn4 z -> happyReturn z; _other -> notHappyAtAll })

happySeq = happyDontSeq


parseError :: Token -> ParserMonad a 
parseError t = failParserMonad ("Error: Parser did not expect token \"" ++ show t ++ "\"")


data Token = TokenSum
           | TokenSeq  
           | TokenString String
           | TokenIndependence
           | TokenUntilFormula
           | TokenOB 
           | TokenCB 
           | TokenOSB 
           | TokenCSB 
           | TokenOA
           | TokenCA
           | TokenOr
           | TokenImplies
           | TokenColon
           | TokenBool
           | TokenPlus
           | TokenPower
           | TokenPlusPlus
           | TokenMinus
           | TokenDivide
           | TokenMultiply
           | TokenThen
           | TokenElse
           | TokenActions
           | TokenComma
           | TokenEqual
           | TokenNotEqual
           | TokenNot
           | TokenPSum
           | TokenDotDot
           | TokenHide
           | TokenRename
           | TokenEncap
           | TokenAnd
           | TokenComm
           | TokenNoComm
           | TokenReach
           | TokenReachCondition
           | TokenConstant
           | TokenFormula
           | TokenType
           | TokenGlobal
           | TokenFunction
           | TokenNat
           | TokenQueue
           | TokenInit
           | TokenSmaller
           | TokenGreater
           | TokenSmallerEq
           | TokenGreaterEq
           | TokenProbDef
           | TokenBecomes
           | TokenEOF
instance Show Token where
  show x =
   case x of 
     TokenSum -> "sum("
     TokenSeq -> "."
     TokenString s -> s
     TokenIndependence -> "||"
     TokenUntilFormula -> "U"
     TokenOB  -> "("
     TokenCB  -> ")"
     TokenOSB -> "["
     TokenCSB -> "]"
     TokenOA  -> "{"
     TokenCA  -> "}"
     TokenOr -> "|"
     TokenImplies -> "=>"
     TokenColon -> ":"
     TokenBool -> "Bool"
     TokenPlus -> "+"
     TokenPower -> "^"
     TokenPlusPlus -> "++"
     TokenMinus -> "-"
     TokenDivide -> "/"
     TokenMultiply -> "*"
     TokenThen -> "<|"
     TokenElse -> "|>"
     TokenActions -> "actions"
     TokenComma -> ","
     TokenEqual -> "="
     TokenNotEqual -> "!="
     TokenNot -> "!"
     TokenPSum -> "psum"
     TokenDotDot -> ".."
     TokenHide -> "hide"
     TokenGlobal -> "global"
     TokenEncap -> "encap"
     TokenAnd -> "&"
     TokenComm -> "comm"
     TokenNoComm -> "nocomm"
     TokenReach -> "reach"
     TokenReachCondition -> "reachCondition"
     TokenConstant -> "constant"
     TokenFormula -> "formula"
     TokenType -> "type"
     TokenFunction -> "function"
     TokenQueue -> "Queue"
     TokenNat -> "Nat"
     TokenInit -> "init"
     TokenSmaller -> "<"
     TokenGreater -> ">"
     TokenSmallerEq -> "<="
     TokenGreaterEq -> ">="
     TokenProbDef -> "->"
     TokenBecomes -> ":="
     TokenEOF -> "EOF"

-- lexer :: String -> [Token] 
lexer :: (Token -> ParserMonad a) -> ParserMonad a
lexer cont ('\n':cs) = \line -> lexer cont cs (line + 1)
lexer cont ('\r':cs) = \line -> lexer cont cs line
lexer cont [] = cont TokenEOF [] 
lexer cont ('-':'-':cs) = lexComment cont (cs)
lexer cont ('.':'.':cs) = cont TokenDotDot cs
lexer cont ('i':'n':'i':'t':' ':cs) = cont TokenInit cs
lexer cont ('i':'n':'i':'t':'\n':cs) = cont TokenInit ('\n':cs)
lexer cont ('i':'n':'i':'t':'\r':cs) = cont TokenInit ('\r':cs)
lexer cont ('i':'n':'i':'t':'\t':cs) = cont TokenInit ('\t':cs)
lexer cont ('u':'n':'t':'i':'l':' ':cs) = cont TokenUntilFormula cs
lexer cont ('u':'n':'t':'i':'l':'\n':cs) = cont TokenUntilFormula ('\n':cs)
lexer cont ('u':'n':'t':'i':'l':'\r':cs) = cont TokenUntilFormula ('\r':cs)
lexer cont ('u':'n':'t':'i':'l':'\t':cs) = cont TokenUntilFormula ('\t':cs)
lexer cont ('b':'o':'o':'l':' ':cs) = cont TokenBool cs
lexer cont ('b':'o':'o':'l':'\n':cs) = cont TokenBool ('\n':cs)
lexer cont ('b':'o':'o':'l':'\r':cs) = cont TokenBool ('\r':cs)
lexer cont ('b':'o':'o':'l':'\t':cs) = cont TokenBool ('\t':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':' ':cs) = cont TokenActions cs
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\n':cs) = cont TokenActions ('\n':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\r':cs) = cont TokenActions ('\r':cs)
lexer cont ('o':'b':'s':'e':'r':'v':'e':'\t':cs) = cont TokenActions ('\t':cs)
lexer cont ('h':'i':'d':'e':' ':cs) = cont TokenHide cs
lexer cont ('h':'i':'d':'e':'\n':cs) = cont TokenHide ('\n':cs)
lexer cont ('h':'i':'d':'e':'\r':cs) = cont TokenHide ('\r':cs)
lexer cont ('h':'i':'d':'e':'\t':cs) = cont TokenHide ('\t':cs)
lexer cont ('h':'i':'d':'e':'(':cs) = cont TokenHide ('(':cs)

lexer cont ('g':'l':'o':'b':'a':'l':' ':cs) = cont TokenGlobal cs
lexer cont ('g':'l':'o':'b':'a':'l':'\n':cs) = cont TokenGlobal ('\n':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'\r':cs) = cont TokenGlobal ('\r':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'\t':cs) = cont TokenGlobal ('\t':cs)
lexer cont ('g':'l':'o':'b':'a':'l':'(':cs) = cont TokenGlobal ('(':cs)

lexer cont ('s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenSum cs
lexer cont ('p':'s':'u':'m':cs) | not (isAlpha (head cs)) = cont TokenPSum cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':' ':cs) = cont TokenConstant cs
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\n':cs) = cont TokenConstant ('\n':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\r':cs) = cont TokenConstant ('\r':cs)
lexer cont ('c':'o':'n':'s':'t':'a':'n':'t':'\t':cs) = cont TokenConstant ('\t':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':' ':cs) = cont TokenFormula cs
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\n':cs) = cont TokenFormula ('\n':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\r':cs) = cont TokenFormula ('\r':cs)
lexer cont ('f':'o':'r':'m':'u':'l':'a':'\t':cs) = cont TokenFormula ('\t':cs)
lexer cont ('t':'y':'p':'e':' ':cs) = cont TokenType cs
lexer cont ('t':'y':'p':'e':'\n':cs) = cont TokenType ('\n':cs)
lexer cont ('t':'y':'p':'e':'\r':cs) = cont TokenType ('\r':cs)
lexer cont ('t':'y':'p':'e':'\t':cs) = cont TokenType ('\t':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':' ':cs) = cont TokenFunction cs
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\n':cs) = cont TokenFunction ('\n':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\r':cs) = cont TokenFunction ('\r':cs)
lexer cont ('f':'u':'n':'c':'t':'i':'o':'n':'\t':cs) = cont TokenFunction ('\t':cs)

lexer cont ('Q':'u':'e':'u':'e':' ':cs) = cont TokenQueue cs
lexer cont ('Q':'u':'e':'u':'e':'\n':cs) = cont TokenQueue ('\n':cs)
lexer cont ('Q':'u':'e':'u':'e':'\r':cs) = cont TokenQueue ('\r':cs)
lexer cont ('Q':'u':'e':'u':'e':'\t':cs) = cont TokenQueue ('\t':cs)

lexer cont ('N':'a':'t':' ':cs) = cont TokenNat cs
lexer cont ('N':'a':'t':'\n':cs) = cont TokenNat ('\n':cs)
lexer cont ('N':'a':'t':'\r':cs) = cont TokenNat ('\r':cs)
lexer cont ('N':'a':'t':'\t':cs) = cont TokenNat ('\t':cs)


lexer cont ('e':'n':'c':'a':'p':' ':cs) = cont TokenEncap cs
lexer cont ('e':'n':'c':'a':'p':'\n':cs) = cont TokenEncap ('\n':cs)
lexer cont ('e':'n':'c':'a':'p':'\r':cs) = cont TokenEncap ('\r':cs)
lexer cont ('e':'n':'c':'a':'p':'\t':cs) = cont TokenEncap ('\t':cs)
lexer cont ('e':'n':'c':'a':'p':'(':cs) = cont TokenEncap ('(':cs)
lexer cont ('r':'e':'n':'a':'m':'e':' ':cs) = cont TokenRename cs
lexer cont ('r':'e':'n':'a':'m':'e':'\n':cs) = cont TokenRename ('\n':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\r':cs) = cont TokenRename ('\r':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'\t':cs) = cont TokenRename ('\t':cs)
lexer cont ('r':'e':'n':'a':'m':'e':'(':cs) = cont TokenRename ('(':cs)
lexer cont ('n':'o':'c':'o':'m':'m':' ':cs) = cont TokenNoComm cs
lexer cont ('n':'o':'c':'o':'m':'m':'\n':cs) = cont TokenNoComm ('\n':cs)
lexer cont ('n':'o':'c':'o':'m':'m':'\r':cs) = cont TokenNoComm ('\r':cs)
lexer cont ('n':'o':'c':'o':'m':'m':'\t':cs) = cont TokenNoComm ('\t':cs)
lexer cont ('r':'e':'a':'c':'h':' ':cs) = cont TokenReach cs
lexer cont ('r':'e':'a':'c':'h':'\n':cs) = cont TokenReach ('\n':cs)
lexer cont ('r':'e':'a':'c':'h':'\r':cs) = cont TokenReach ('\r':cs)
lexer cont ('r':'e':'a':'c':'h':'\t':cs) = cont TokenReach ('\t':cs)
lexer cont ('r':'e':'a':'c':'h':'C':'o':'n':'d':'i':'t':'i':'o':'n':' ':cs) = cont TokenReachCondition cs
lexer cont ('r':'e':'a':'c':'h':'C':'o':'n':'d':'i':'t':'i':'o':'n':'\n':cs) = cont TokenReachCondition ('\n':cs)
lexer cont ('r':'e':'a':'c':'h':'C':'o':'n':'d':'i':'t':'i':'o':'n':'\r':cs) = cont TokenReachCondition ('\r':cs)
lexer cont ('r':'e':'a':'c':'h':'C':'o':'n':'d':'i':'t':'i':'o':'n':'\t':cs) = cont TokenReachCondition ('\t':cs)
lexer cont ('c':'o':'m':'m':' ':cs) = cont TokenComm cs
lexer cont ('c':'o':'m':'m':'\n':cs) = cont TokenComm ('\n':cs)
lexer cont ('c':'o':'m':'m':'\r':cs) = cont TokenComm ('\r':cs)
lexer cont ('c':'o':'m':'m':'\t':cs) = cont TokenComm ('\t':cs)
lexer cont (c:cs) 
      | isSpace c = lexer cont cs 
      | (\x -> isDigit x) c = lexNumber cont (c:cs)
      | (\x -> isAlpha x) c = lexString cont (c:cs)
lexer cont ('.':cs) = cont TokenSeq cs 
lexer cont ('|':'|':cs) = cont TokenIndependence cs
lexer cont ('|':'>':cs) = cont TokenElse cs 
lexer cont ('<':'|':cs) = cont TokenThen cs 
lexer cont ('&':cs) = cont TokenAnd cs 
lexer cont ('<':'=':cs) = cont TokenSmallerEq cs 
lexer cont ('>':'=':cs) = cont TokenGreaterEq cs 
lexer cont ('<':cs) = cont TokenSmaller cs 
lexer cont ('>':cs) = cont TokenGreater cs 
lexer cont ('(':cs) = cont TokenOB cs 
lexer cont ('|':cs) = cont TokenOr cs 
lexer cont (')':cs) = cont TokenCB cs 
lexer cont ('[':cs) = cont TokenOSB cs 
lexer cont (']':cs) = cont TokenCSB cs 
lexer cont ('{':cs) = cont TokenOA cs 
lexer cont ('}':cs) = cont TokenCA cs 
lexer cont ('+':'+':cs) = cont TokenPlusPlus cs
lexer cont ('-':'>':cs) = cont TokenProbDef cs
lexer cont (',':cs) = cont TokenComma cs
lexer cont ('+':cs) = cont TokenPlus cs
lexer cont ('^':cs) = cont TokenPower cs
lexer cont ('-':cs) = cont TokenMinus cs
lexer cont ('!':'=':cs) = cont TokenNotEqual cs
lexer cont ('!':cs) = cont TokenNot cs
lexer cont ('/':cs) = cont TokenDivide cs
lexer cont ('*':cs) = cont TokenMultiply cs
lexer cont (':':'=':cs) = cont TokenBecomes cs
lexer cont (':':cs) = cont TokenColon cs
lexer cont ('=':'>':cs) = cont TokenImplies cs
lexer cont ('=':cs) = cont TokenEqual cs
lexer cont s = \line -> error("Error: Lexer could not deal with character \"" ++ [head s] ++ "\" on line " ++ show line)

lexComment cont cs = lexer cont rest
  where
    rest = dropWhile (\x -> x /= '\n' && x /= '\r') cs
lexString cont cs = cont (TokenString var) rest
  where
    (var,rest) = getString cs
lexNumber cont cs = cont (TokenString var) rest
  where
    (var,rest) = getNumber cs

getString [] = ("","")
getString ('.':'.':cs) = ("", '.':'.':cs)
getString (c:cs) | elem c "\n\t\r ',()&=*.<>+{}^:-!?/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getString cs
    str   = c:s
    rest  = r

getNumber [] = ("","")
getNumber ('.':'.':cs) = ("", '.':'.':cs)
getNumber (c:cs) | elem c "\n\t\r ',()&=*<>+_{}:^-!?/|\"[]" = ("", c:cs)
                 | otherwise = (str,rest)
  where
    (s,r) = getNumber cs
    str   = c:s
    rest  = r
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
{-# LINE 1 "<built-in>" #-}
{-# LINE 1 "<command-line>" #-}
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- Id: GenericTemplate.hs,v 1.26 2005/01/14 14:47:22 simonmar Exp 

{-# LINE 30 "templates/GenericTemplate.hs" #-}








{-# LINE 51 "templates/GenericTemplate.hs" #-}

{-# LINE 61 "templates/GenericTemplate.hs" #-}

{-# LINE 70 "templates/GenericTemplate.hs" #-}

infixr 9 `HappyStk`
data HappyStk a = HappyStk a (HappyStk a)

-----------------------------------------------------------------------------
-- starting the parse

happyParse start_state = happyNewToken start_state notHappyAtAll notHappyAtAll

-----------------------------------------------------------------------------
-- Accepting the parse

-- If the current token is (1), it means we've just accepted a partial
-- parse (a %partial parser).  We must ignore the saved token on the top of
-- the stack in this case.
happyAccept (1) tk st sts (_ `HappyStk` ans `HappyStk` _) =
	happyReturn1 ans
happyAccept j tk st sts (HappyStk ans _) = 
	 (happyReturn1 ans)

-----------------------------------------------------------------------------
-- Arrays only: do the next action

{-# LINE 148 "templates/GenericTemplate.hs" #-}

-----------------------------------------------------------------------------
-- HappyState data type (not arrays)



newtype HappyState b c = HappyState
        (Int ->                    -- token number
         Int ->                    -- token number (yes, again)
         b ->                           -- token semantic value
         HappyState b c ->              -- current state
         [HappyState b c] ->            -- state stack
         c)



-----------------------------------------------------------------------------
-- Shifting a token

happyShift new_state (1) tk st sts stk@(x `HappyStk` _) =
     let (i) = (case x of { HappyErrorToken (i) -> i }) in
--     trace "shifting the error token" $
     new_state i i tk (HappyState (new_state)) ((st):(sts)) (stk)

happyShift new_state i tk st sts stk =
     happyNewToken new_state ((st):(sts)) ((HappyTerminal (tk))`HappyStk`stk)

-- happyReduce is specialised for the common cases.

happySpecReduce_0 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_0 nt fn j tk st@((HappyState (action))) sts stk
     = action nt j tk st ((st):(sts)) (fn `HappyStk` stk)

happySpecReduce_1 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_1 nt fn j tk _ sts@(((st@(HappyState (action))):(_))) (v1`HappyStk`stk')
     = let r = fn v1 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_2 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_2 nt fn j tk _ ((_):(sts@(((st@(HappyState (action))):(_))))) (v1`HappyStk`v2`HappyStk`stk')
     = let r = fn v1 v2 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happySpecReduce_3 i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happySpecReduce_3 nt fn j tk _ ((_):(((_):(sts@(((st@(HappyState (action))):(_))))))) (v1`HappyStk`v2`HappyStk`v3`HappyStk`stk')
     = let r = fn v1 v2 v3 in
       happySeq r (action nt j tk st sts (r `HappyStk` stk'))

happyReduce k i fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyReduce k nt fn j tk st sts stk
     = case happyDrop (k - ((1) :: Int)) sts of
	 sts1@(((st1@(HappyState (action))):(_))) ->
        	let r = fn stk in  -- it doesn't hurt to always seq here...
       		happyDoSeq r (action nt j tk st1 sts1 r)

happyMonadReduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonadReduce k nt fn j tk st sts stk =
        happyThen1 (fn stk tk) (\r -> action nt j tk st1 sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk

happyMonad2Reduce k nt fn (1) tk st sts stk
     = happyFail (1) tk st sts stk
happyMonad2Reduce k nt fn j tk st sts stk =
       happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
       where (sts1@(((st1@(HappyState (action))):(_)))) = happyDrop k ((st):(sts))
             drop_stk = happyDropStk k stk





             new_state = action


happyDrop (0) l = l
happyDrop n ((_):(t)) = happyDrop (n - ((1) :: Int)) t

happyDropStk (0) l = l
happyDropStk n (x `HappyStk` xs) = happyDropStk (n - ((1)::Int)) xs

-----------------------------------------------------------------------------
-- Moving to a new state after a reduction

{-# LINE 246 "templates/GenericTemplate.hs" #-}
happyGoto action j tk st = action j j tk (HappyState action)


-----------------------------------------------------------------------------
-- Error recovery ((1) is the error token)

-- parse error if we are in recovery and we fail again
happyFail (1) tk old_st _ stk@(x `HappyStk` _) =
     let (i) = (case x of { HappyErrorToken (i) -> i }) in
--	trace "failing" $ 
        happyError_ i tk

{-  We don't need state discarding for our restricted implementation of
    "error".  In fact, it can cause some bogus parses, so I've disabled it
    for now --SDM

-- discard a state
happyFail  (1) tk old_st (((HappyState (action))):(sts)) 
						(saved_tok `HappyStk` _ `HappyStk` stk) =
--	trace ("discarding state, depth " ++ show (length stk))  $
	action (1) (1) tk (HappyState (action)) sts ((saved_tok`HappyStk`stk))
-}

-- Enter error recovery: generate an error token,
--                       save the old token and carry on.
happyFail  i tk (HappyState (action)) sts stk =
--      trace "entering error recovery" $
	action (1) (1) tk (HappyState (action)) sts ( (HappyErrorToken (i)) `HappyStk` stk)

-- Internal happy errors:

notHappyAtAll :: a
notHappyAtAll = error "Internal Happy error\n"

-----------------------------------------------------------------------------
-- Hack to get the typechecker to accept our action functions







-----------------------------------------------------------------------------
-- Seq-ing.  If the --strict flag is given, then Happy emits 
--	happySeq = happyDoSeq
-- otherwise it emits
-- 	happySeq = happyDontSeq

happyDoSeq, happyDontSeq :: a -> b -> b
happyDoSeq   a b = a `seq` b
happyDontSeq a b = b

-----------------------------------------------------------------------------
-- Don't inline any functions from the template.  GHC has a nasty habit
-- of deciding to inline happyGoto everywhere, which increases the size of
-- the generated parser quite a bit.

{-# LINE 312 "templates/GenericTemplate.hs" #-}
{-# NOINLINE happyShift #-}
{-# NOINLINE happySpecReduce_0 #-}
{-# NOINLINE happySpecReduce_1 #-}
{-# NOINLINE happySpecReduce_2 #-}
{-# NOINLINE happySpecReduce_3 #-}
{-# NOINLINE happyReduce #-}
{-# NOINLINE happyMonadReduce #-}
{-# NOINLINE happyGoto #-}
{-# NOINLINE happyFail #-}

-- end of Happy Template.
